#ifndef CPUDATA_H
#define CPUDATA_H

#include <vector>
#include <string>
#include <set>

#include "log.h"

class Device;

struct Gas
{
    static const char * T_RO;
    float Ro;// плотность
    static const char * T_PR;
    float P;// давление
    static const char * T_VX;
    float Vx;// скорость по х
    static const char * T_VY;
    float Vy;// скорость по у
    static const char * T_VS;
    float Vs;// вязкость
    static const char * T_SC;
    float Sc;//схема
    static const char * T_CS;
    float Cs;// линейность цветов
    static const char * T_R1;
    float R1;// цвет красный
    static const char * T_G1;
    float G1;// цвет зеленый
    static const char * T_B1;
    float B1;// цвет синий
    static const char * T_R2;
    float R2;// цвет красный
    static const char * T_G2;
    float G2;// цвет зеленый
    static const char * T_B2;
    float B2;// цвет синий
    static const char * T_R3;
    float R3;// цвет красный
    static const char * T_G3;
    float G3;// цвет зеленый
    static const char * T_B3;
    float B3;// цвет синий
    static const char * T_FULL_TIME_LEFT;
    float FullTimeLeft;// цвет синий
    static const char * T_MARK_TIME_LEFT;
    float MarkTimeLeft;// цвет синий
    Gas();
    void check();
};

std::vector<float> serilizeGas(std::vector<std::vector<Gas> > gas);
std::vector<std::vector<Gas> > deserilizeGas(std::vector<float> input);

struct CpuGravityBody
{
    static const char * BODY_ID;
    std::string id;
    static const char * BODY_X;
    float x;
    static const char * BODY_Y;
    float y;
    static const char * BODY_VX;
    float vx;
    static const char * BODY_VY;
    float vy;
    static const char * BODY_MASS;
    float mass;
    static const char * BODY_RADIUS;
    float radius;

    float gasVx;
    float gasVy;
    float gasRo;

    static const char * BODY_GAS_V;
    float gasV;//множитель при скорости газа
    static const char * BODY_GAS_M;
    float gasM;//множитель при массе газа

    static const char * BODY_BODY_V;
    float bodyV;//множитель при собственной скорости газа

    static const char * BODY_LIFETIME;
    float lifeTime;
    static const char * BODY_LIFEPATH;
    float lifePath;

    CpuGravityBody(std::string Id):id(Id),
        lifeTime(1e35f),
        lifePath(1e35f),
        x(0),
        y(0),
        vx(0),
        vy(0),
        mass(1.0f),
        radius(1.0f),
        bodyV(1.0f),
        gasV(0.0f),
        gasM(0.0f),
        gasRo(0.0f),
        gasVx(0.0f),
        gasVy(0.0f){}

    float getMass();
    float getVx();
    float getVy();
};

class CpuData
{
    std::vector<std::vector<Gas> > _gas;
    std::vector<CpuGravityBody> _body;

    float _dt;
    float _ndt;
    long double _time;
    int _step;
    pthread_mutex_t _mutex;

    float _frameTime;
    int _frameStep;

public:
    CpuData(); 
    ~CpuData();
    void init(std::vector<std::vector<Gas> > gas);
    float getDt();
    void setDt();
    void setNextDt(float);
    float getTime();
    int getStep();
    bool isFrame();
    bool makeFrame( float timeFrame,int enterFrame);
    Position getSize();
    Position getZeroPosition();
    std::vector<std::vector<Gas> > & getGas();
    Gas getGas(Position position);
    bool setGas(Gas gas, Position position);
    bool addBody(CpuGravityBody body);
    bool removeBody(std::string id);
    bool bodyStep(float dt);
    bool makeConstBody(GpuGravityBody * body);
    int getBodyCount();
    CpuGravityBody * getBody(int n);

    void operator=(Device * data);
};

#endif // CPUDATA_H
