#ifndef device_H
#define device_H

#include "gpudata.h"
#include "cpudata.h"
#include "scenario.h"
#include "outdata.h"

#include <pthread.h>
#include <memory>
#include <vector>

class Device
{
    Device * _upper;
    Device * _downer;
    std::auto_ptr<GpuData> _gpuData;
    std::auto_ptr<GpuData> _cpuBufer;
    std::auto_ptr<GpuData> _upMark;
    std::auto_ptr<GpuData> _dwMark;
    CpuData * _cpuData;
    Scenario * _scenario;
    int _gpuId;
    Position _stringMarks;
    bool _transponate;
    pthread_t _thread;
    pthread_attr_t _attr;

    static void * thread_stepTo(void * arg);
    static void * thread_init(void * arg);
    static void * thread_update(void * arg);
    static void * thread_getGpuGas(void * arg);
    static void * thread_getColors(void * arg);
    static void * thread_gpuFree(void * arg);

    void getMarks();
    void getGasCorrectedMarks();
    void getGasSolvedMarks();
    void getDataCorrectedMarks();

    void setMarks(std::auto_ptr<GpuData> * up=0, std::auto_ptr<GpuData> * dw=0);
    void setGasCorrectedMarks(std::auto_ptr<GpuData> * up=0, std::auto_ptr<GpuData> * dw=0);
    void setGasSolvedMarks(std::auto_ptr<GpuData> * up=0, std::auto_ptr<GpuData> * dw=0);
    void setDataCorrectedMarks(std::auto_ptr<GpuData> * up=0, std::auto_ptr<GpuData> * dw=0);
    int getHeight();

public:
    Device();
    Device(const Device & in);
    void operator=(const Device & in){}
    ~Device();
    bool init(CpuData * cpuData, Scenario * scenario, int gpuId, Position stringMarks, Device * upper=0, Device * downer=0);
    bool update();
    bool step();
    void wait();
    void getGpuGas(std::vector<std::vector<Gas> > & gas);
    void getColors(std::vector<std::vector<float> > & colors);
    int getGpuId();
    std::auto_ptr<GpuData> * getUpMark();
    std::auto_ptr<GpuData> * getDownMark();
    static std::vector<std::string> getGpusInfo();
    static GpuInfo getGpuInfo(int id);
    static bool setDeviceCount(int count);
};

#endif // device_H
