#include "filemanager.h"
#include <sstream>
#include <iostream>
#include <stdlib.h>
#include <fstream>
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include "log.h"
#include "EasyBMP/EasyBMP.h"

std::vector<std::string> split_string(std::string text, const char splitter, bool twoPart)
{
    std::vector<std::string> out;
    std::string::size_type pos=0;
    while(pos<text.size())
    {
        std::string::size_type split_pos = text.find_first_of(splitter,pos);
        if(split_pos == std::string::npos||(twoPart&&out.size()>0))
        {
            out.push_back(text.substr(pos,text.size()-pos));
            break;
        }
        else
        {
            if(split_pos-pos > 0)
                out.push_back(text.substr(pos,split_pos-pos));
        }

        pos=split_pos + 1;
    }
    return out;
}

std::string symbol_eraser(std::string text, const char symbol)
{
    for(std::string::size_type pos = text.find_first_of(symbol); pos != std::string::npos; pos = text.find_first_of(symbol))
        text.erase(pos,1);
    return text;
}

float stringToFloat(std::string input)
{
    return atof(input.c_str());
}

std::string floatToString(float input)
{
    std::stringstream ss;
    ss << input;
    return ss.str();
}

int stringToInt(std::string input)
{
    return atoi(input.c_str());
}

std::string intToString(int input)
{
    std::stringstream ss;
    ss << input;
    return ss.str();
}

FileManager::FileManager()
{

}

std::vector<std::string> FileManager::read(std::string path)
{
    std::vector<std::string> out;

    std::ifstream ifs(path.c_str());

    if (!ifs.is_open())
    {
        logger(std::string("can't open file : ")+path);
        return out;
    }

    for (std::string line; std::getline(ifs, line); )
    {
        if((line.size()>=2&&line.at(0)!='/'&&line.at(1)!='/')||(line.size()==1))
            out.push_back(line);
    }

    ifs.close();

    return out;
}

std::vector<float> FileManager::load(std::string path)
{
    std::vector<float> out;

    FILE * file = fopen(path.c_str(),"rb");

    if(!file)
    {
        logger(std::string("can't open file : ")+path);
        return std::vector<float>();
    }

    struct stat fi;
    stat(path.c_str(),&fi);
    printf("file size: %d",static_cast<int>(fi.st_size));

    if(fi.st_size<=0L)
    {
        logger(std::string("wrong size of file : ")+path);
        return std::vector<float>();
    }

    out.resize(fi.st_size/sizeof(float));

    if(fread(&out[0],sizeof(float),fi.st_size/sizeof(float),file)!=fi.st_size/sizeof(float))
    {
        logger(std::string("can't read file : ")+path);
        return std::vector<float>();
    }

    fclose(file);

    return out;
}

bool FileManager::write(std::string path, std::vector<std::string> data)
{
    std::ofstream ofs(path.c_str());

    if (!ofs.is_open())
    {
        logger(std::string("can't open file : ")+path);
        return false;
    }

    for(std::vector<std::string>::iterator it = data.begin(); it != data.end(); ++it)
    {
        ofs << *it << std::endl;
    }

    ofs.close();

    return true;
}

bool FileManager::save(std::string path, std::vector<float> data)
{
    FILE * file = fopen(path.c_str(),"wb");

    if(!file)
    {
        logger(std::string("can't create file : ")+path);
        return false;
    }

    if(fwrite(&data[0],sizeof(float),data.size(),file)!=data.size())
    {
        logger(std::string("can't write to file : ")+path);
        return false;
    }

    fclose(file);

    return true;
}

bool encoder(std::string path, int frameRate, int scale, bool removeBmp)
{
    try
    {
    if(scale>1)
    {
        BMP bmp;
        int num=1;
        std::string bmpPath=path+"00000000.bmp";
        std::vector<std::vector<RGBApixel> > data;
        while(bmp.ReadFromFile(bmpPath.c_str()))
        {
            if(data.size()!=bmp.TellHeight()/scale)
            {
                data.resize(bmp.TellHeight()/scale);
                for(int i=0;i<data.size();++i)
                {
                    data[i].resize(bmp.TellWidth()/scale);
                }
            }

            for(int s=0;s<data.size();++s)
            {
                for(int c=0;c<data[s].size();++c)
                {
                    int r=0;
                    int g=0;
                    int b=0;
                    for(int i=0;i<scale;i++)
                    {
                        for(int j=0;j<scale;j++)
                        {
                            RGBApixel px=bmp.GetPixel(c*scale+j,s*scale+i);
                            r+=px.Red;
                            g+=px.Green;
                            b+=px.Blue;
                        }
                    }
                    data[s][c].Red=r/(scale*scale);
                    data[s][c].Green=g/(scale*scale);
                    data[s][c].Blue=b/(scale*scale);
                }
            }

            bmp.SetSize(data[0].size(),data.size());

            for(int s=0;s<data.size();++s)
            {
                for(int c=0;c<data[s].size();++c)
                {
                    bmp.SetPixel(c,s,data[s][c]);
                }
            }

            bmp.WriteToFile(bmpPath.c_str());

            std::cout<<bmpPath<<" complete"<<std::endl;

            std::string str=intToString(num++);
            bmpPath=path;
            for(int i=str.size();i<8;++i)
                bmpPath += '0';
            bmpPath += str + std::string(".bmp");
        }
    }
    int outNumber=0;
    FILE * file=fopen((path+"out.mkv").c_str(),"rb");
    while(file)
    {
        fclose(file);
        outNumber++;
        file=fopen((path+"out" + intToString(outNumber)+".mkv").c_str(),"rb");
    }
    if(chdir(path.c_str()))
    {
        logger("can't change directory");
        return true;
    }

    remove("/tmp/out.mkv");

    std::string command("ffmpeg -r ");
    command += intToString(frameRate);
    command += " -y -i %08d.bmp -c:v libx264 -qp 16 -r ";
    command += intToString(frameRate);
    command += " -me_method tesa -threads 32 -g ";
    command += intToString(frameRate);
    command +=" /tmp/out.mkv";
    system(command.c_str());

    file=0;
    while(!file)
    {
        sleep(5);
        file=fopen("/tmp/out.mkv","rb");
    }

    fclose(file);
    system((std::string("mv /tmp/out.mkv ")+path+"out"+(outNumber>0?intToString(outNumber):"")+".mkv").c_str());
    if(removeBmp)
    {
        int num=1;
        std::string bmpPath=path+"00000000.bmp";
        file=fopen(bmpPath.c_str(),"rb");
        while(file)
        {
            fclose(file);
            remove(bmpPath.c_str());
            std::cout<<bmpPath<<" removed"<<std::endl;
            std::string str=intToString(num++);
            bmpPath=path;
            for(int i=str.size();i<8;++i)
                bmpPath += '0';
            bmpPath += str + std::string(".bmp");
            file=fopen(bmpPath.c_str(),"rb");
        }
    }
    }
    catch(...)
    {
        return true;
    }

    return false;
}
