#include "gpu.h"
#include <stdio.h>
#include <iostream>
#include <string>

#define gpuMaxBlockSize 1024
#define gpuMaxBlockSide 32

#define maxRo 1e12f
#define maxT 2097152.0f
#define maxV 2048.0f
#define maxRoVisc 2048.0f
#define minRo 1e-6f
#define minT 1e-6f
#define minP 1e-6f
#define minCs 1e-6f
#define minSc 0.0f
#define minColor 0.0f
#define maxCs 2048.0f
#define maxSc 4.0f
#define maxColor 256.0f


#define sign(a) ((a)>0.0f?(1.0f):(-1.0f))

#define safe(x) (sign(x) * max(fabs(x),1e-12f))
#define usafe(x) (max((x),1e-12f))
#define dsafe(x,y) (max(max((x),(y)),1e-12f))
#define nsafe(x) (isfinite((x))?(x):0.0f)

#define limiter(x,minVal,maxVal) (max(minVal,min(maxVal,x)))

//linear block

#define initPosition \
int width=data.width;\
int xp = blockIdx.x * blockDim.x + threadIdx.x + width;

#define initPositionL \
int xl;\
if(data.transponate)\
{\
    xl = xp - width;\
}\
else\
{\
    xl = xp - ((xp%width > 0) ? 1 : 0);\
}

#define initPositionR \
int xr;\
if(data.transponate)\
{\
    xr = xp + width;\
}\
else\
{\
    xr = xp + ((xp%width < (width - 1)) ? 1 : 0);\
}

#define initPositionLR \
int xl, xr;\
if(data.transponate)\
{\
    xl = xp - width;\
    xr = xp + width;\
}\
else\
{\
    int x = xp%width;\
    xl = xp - ((x > 0) ? 1 : 0);\
    xr = xp + ((x < (width - 1)) ? 1 : 0);\
}


#define initPositionXY \
int width=data.width;\
int xp = blockIdx.x * blockDim.x + threadIdx.x+width;\
int x = xp%width;\
int xl = xp - ((x > 0) ? 1 : 0);\
int xr = xp + ((x < (width - 1)) ? 1 : 0);\
int xu = xp - width;\
int xd = xp + width;\
if(data.transponate)\
{\
x=xl; xl=xu; xu=x;\
x=xr; xr=xd; xd=x;\
}

#define gm 1.4f
#define gm1 0.4f
//(gm - 1.0f)
#define g1 0.142857143f
//((gm - 1.0f)/(2.0f*gm))
#define g2 0.857142857f
//((gm + 1.0f)/(2.0f*gm))
#define g3 7.0f
//(2.0f*gm/(gm - 1.0f))
#define g4 5.0f
//(2.0f/(gm - 1.0f))
#define g5 0.833333333f
//(2.0f/(gm + 1.0f))
#define g6 0.166666667f
//((gm - 1.0f)/(gm + 1.0f))
#define g7 0.2f
//((gm - 1.0f)/2.0f)

__constant__ GpuGravityBody bodys[4096];

__device__ float mhs(float a,float b)
{
    return ((a)*(b)>0.0f?(2.0f*(a)*(b)/(a+b)):0.0f);
}

__device__ float smhs(float a,float b)
{
    return ((a)*(b)>0.0f?(min(2.0f*min(fabs(a),fabs(b)),max(fabs(a),fabs(b)))*sign(a)):0.0f);
}

__device__ float minmod(float a,float b,float c)
{
    return ((c<=2.0f)?(mhs(a,b)*(c-1.0f)):((c<3.0f)?(mhs(a,b)*(3.0f-c)+smhs(a,b)*(c-2.0f)):(smhs(a,b))));
}

__device__ float correctG(float l, float c, float r, float La2, float scheme)
{
    float out=c + 0.5f * ((La2<0.0f?-1.0f:1.0f) - La2) * minmod(c - l,r - c,scheme);
    return nsafe(out);
}

__global__ void gpuGasCorrect(GpuGasCorrectData data)
{
    initPosition
    initPositionLR
    float dt, ro, p, u, c2, z, c, q, U1, U2, U3, V1, V2, V3, La1, La2, La3, Rol, Pl, Vl, Ro, P, V, Ror, Pr, Vr,scheme;

    Rol=data.Ro0[xl];
    Pl=data.P0[xl];
    Vl=data.Vx0[xl];

    Ro=data.Ro0[xp];
    P=data.P0[xp];
    V=data.Vx0[xp];

    Ror=data.Ro0[xr];
    Pr=data.P0[xr];
    Vr=data.Vx0[xr];

    scheme=data.Sc0[xp];
    dt=data.dt;

    ro=min(min(Rol,Ror),Ro);
    p=min(min(Pl,Pr),P);

    V1=ro/usafe(max(max(Rol,Ror),Ro));
    U1=p/usafe(max(max(Pl,Pr),P));
    u=V1*U1/usafe(V1+U1);

    z=min(scheme,4.0f+log10f(max(1e-4f,u)));
    scheme=2.0f*scheme*z/usafe(scheme+z);

    La2 = V * dt;
    if(P>=0.0f&&Ro>=0.0f)
    {
        data.Sc1[xp]=scheme;
        data.Cs1[xp]=La2;
    }

    if(scheme<=1.0f||
            ro<-minRo||p<-minP||
            Ro<minRo||P<minP)
    {
        data.Ro1[xp]=Ro;
        data.P1[xp]=P;
        data.Vx1[xp]=data.Vx0[xp];
        data.Vy1[xp]=data.Vy0[xp];
        data.Vs1[xp]=data.Vs0[xp];
        return;
    }

    U1 = Ro - Rol;
    V1 = Ror - Ro;
    U2 = P - Pl;
    V2 = Pr - P;
    U3 = V - Vl;
    V3 = Vr - V;
    ro = Ro;
    p = P;
    u = V;
    c2 = usafe(gm * P / usafe(Ro));
    c = sqrtf(c2);
    p = 0.5f * U2 / c2;
    q = 0.5f * U3 * ro / c;
    z = U1;
    U1 = p - q;
    U3 = p + q;
    U2 = z - p - p;
    p = 0.5f * V2 / c2;
    q = 0.5f * V3 * ro / c;
    z = V1;
    V1 = p - q;
    V3 = p + q;
    V2 = z - p - p;
    La1 = (u - c) * dt;
    La3 = (u + c) * dt;
    U1 = 0.5f * ((La1<0.0f?-1.0f:1.0f) - La1) * minmod(U1,V1,scheme);
    U2 = 0.5f * ((La2<0.0f?-1.0f:1.0f) - La2) * minmod(U2,V2,scheme);
    U3 = 0.5f * ((La3<0.0f?-1.0f:1.0f) - La3) * minmod(U3,V3,scheme);

    Ro=Ro + U1 + U2 + U3;
    P=P + c2 * (U1 + U3);
    V=V + c * (U3 - U1) / usafe(ro);
    V2=correctG(data.Vy0[xl],data.Vy0[xp],data.Vy0[xr],La2,scheme);
    U2=correctG(data.Vs0[xl],data.Vs0[xp],data.Vs0[xr],La2,scheme);

    if(Ro<minRo)
    {
        Ro=P=V=V2=U2=0.0f;
    }

    data.Ro1[xp]=Ro;
    data.P1[xp]=limiter(nsafe(P),minT*Ro,maxT*Ro);
    data.Vx1[xp]=V;
    data.Vy1[xp]=V2;
    data.Vs1[xp]=U2;
}

__device__ float3 rieman_solver(float rl,float pl,float vl,float rr,float pr,float vr)
{
    float3 out;
    float cl, cr, al, ar, ro, p, v;
    float pratio, Sh, fl, fld, fr, frd, St, c, Pm, Cm;
    bool dirV, dirP;

    cl=usafe(sqrtf(gm*pl/usafe(rl)));
    cr=usafe(sqrtf(gm*pr/usafe(rr)));

    p = max(0.0f,0.5f * (pl + pr) + (vl - vr) * 0.125f * (rl + rr) * (cl + cr));

    Pm = min(pr,pl);
    Cm = max(pl,pr);

    if ((Cm / usafe(Pm)) > 2.0f || Pm > p || p > Cm)
    {
       if (p < Pm)
       {
           Cm = powf(usafe(pl / usafe(pr)), g1);
           Pm = (Cm * vl / cl + vr / cr + g4 * (Cm - 1.0f)) / usafe(Cm / cl + 1.0f / cr);
           al = 1.0f + g7 * (vl - Pm) / cl;
           ar = 1.0f + g7 * (Pm - vr) / cr;
           p = 0.5f * (pl * powf(usafe(al), g3) + pr * powf(usafe(ar), g3));
       }
       else
       {
           al = sqrtf((g5 / usafe(rl)) / usafe(g6 * pl + p));
           ar = sqrtf((g5 / usafe(rr)) / usafe(g6 * pr + p));
           p = (al * pl + ar * pr - (vr - vl)) / usafe(al + ar);
       }
    }

    Pm=10.0f*p;
    for (int i = 0; i < 3 && fabs(p-Pm)/usafe(p+Pm)>1e-3f; i++)
    {
       if (p <= pl)
       {
           pratio = usafe(p / usafe(pl));
           fl = g4 * cl * (powf(pratio, g1) - 1.0f);
           fld = (1.0f / usafe(rl * cl)) * powf(pratio, -g2);
       }
       else
       {
           al = g5 / usafe(rl);
           ar = g6 * pl;
           Sh = sqrtf(al / usafe(ar + p));
           fl = (p - pl) * Sh;
           fld = (1.0f - 0.5f * (p - pl) / usafe(ar + p)) * Sh;
       }

       if (p <= pr)
       {
           pratio = usafe(p / usafe(pr));
           fr = g4 * cr * (powf(pratio, g1) - 1.0f);
           frd = (1.0f / usafe(rr * cr)) * powf(pratio, -g2);
       }
       else
       {
           al = g5 / usafe(rr);
           ar = g6 * pr;
           Sh = sqrtf(al / usafe(ar + p));
           fr = (p - pr) * Sh;
           frd = (1.0f - 0.5f * (p - pr) / usafe(ar + p)) * Sh;
       }
       Pm=p;
       p = max(0.0f,p - (fl + fr + vr - vl) / usafe(fld + frd));
    }

    v = 0.5f * (vl + vr + fr - fl);

    if(pl/usafe(rl)<1e-3f||pr/usafe(rr)<1e-3f||pr>1e6f||pl>1e6f)
    {
        al=rl*(vl-min(vl-cl,vr-cr));
        ar=rr*(max(vl+cl,vr+cr)-vr);
        p=max(0.0f,(pl*ar+pr*al-ar*al*(vr-vl))/safe(al+ar));
        v=(vr*ar+vl*al-pr+pl)/safe(al+ar);
    }

    dirV = (v >= 0.0f);
    dirP = dirV?
                (p <= pl)
                :
                (p > pr) ;
    Pm=dirV&&(dirP - 1.0f)?
                (rl * (p / usafe(pl) + g6) / usafe((p / usafe(pl)) * g6 + 1.0f))
                :
                (rr * (p / usafe(pr) + g6) / usafe((p / usafe(pr)) * g6 + 1.0f));
    Sh = dirV?
                (dirP?
                     (vl - cl)
                     :
                     (v+(vl-v)*rl/safe(rl-Pm)))
                :
                (dirP?
                     (v+(vr-v)*rr/safe(rr-Pm))
                     :
                     (vr+cr));
    Cm = dirV&&dirP?
                cl * powf(usafe(p/usafe(pl)),g1)
                :
                cr * powf(usafe(p/usafe(pr)),g1);
    St = dirV?
                (v-Cm)
                :
                (v+Cm);
    c = dirV&&dirP?
                g5 * (cl + g7*vl)
                :
                g5 * (cr - g7*vr);
    ro=dirV?
            (dirP?
                (0.0f<=Sh?
                    (rl)
                    :
                    (St<0.0f?
                        (rl * powf(usafe(p/usafe(pl)),1.0f/gm))
                        :
                        (rl*powf(usafe(c/cl),g4)))
                )
                :
                (0.0f<=Sh?
                    (rl)
                    :
                    (Pm)
                )
            )
            :
            (dirP?
                (Sh<=0.0f?
                    (rr)
                    :
                    (Pm)
                )
                :
                (Sh<=0.0f?
                    (rr)
                    :
                    (0.0f<=St?
                        (rr * powf(usafe(p/usafe(pr)),1.0f/gm))
                        :
                        (rr * powf(usafe(c/cr),g4))
                    )
                )
            );
    p=dirV?
        (dirP?
            (0.0f<=Sh?
                (pl)
                :
                (St<0.0f?
                    (p)
                    :
                    (pl*powf(usafe(c/cl),g3))
                )
            )
            :
            (0.0f<=Sh?
                (pl)
                :
                (p)
            )
        )
        :
        (dirP?
            (Sh<=0.0f?
                (pr)
                :
                (p)
            )
            :
            (Sh<=0.0f?
                (pr)
                :
                (0.0f<=St?
                    (p)
                    :
                    (pr * powf(usafe(c/cr),g3))
                )
            )
        );
    v=dirV?
          (dirP?
               (0.0f<=Sh?
                    (vl)
                    :
                    (St<0.0f?
                         (v)
                         :
                         (g5 * (cl + g7*vl))
                    )
                )
                :
                (0.0f<=Sh?
                     (vl)
                     :
                     (v)
                )
           )
           :
           (dirP?
                (Sh<=0.0f?
                     (vr)
                     :
                     (v)
                )
                :
                (Sh<=0.0f?
                     (vr)
                     :
                     (0.0f<=St?
                          (v)
                          :
                          (g5 * (-cr + g7*vr))
                     )
                )
           );

    if(ro<=0.0f)
        ro=p=v=0.0f;

    out.x=ro;
    out.y=p;
    out.z=v;

    return out;
}

__global__ void gpuGasRieman(GpuGasRiemanData data)
{
    initPosition
    initPositionL

    xp+=data.shift;
    xl+=data.shift;

    float cro, cp, cvx;
    float lro, lp, lvx;

    cro=data.Ro1[xp];
    lro=data.Ro1[xl];
    cp=data.P1[xp];
    lp=data.P1[xl];

    bool left_solid_neighbor=lro<-minRo;
    bool left_const_neighbor=lp<-minP;
    bool right_solid_neighbor=cro<-minRo;
    bool right_const_neighbor=cp<-minP;

    if((left_solid_neighbor&&right_solid_neighbor)||(left_const_neighbor&&right_const_neighbor))
    {
        return;
    }

    cp=fabs(cp);
    lp=fabs(lp);

    cvx=data.Vx1[xp];
    lvx=data.Vx1[xl];

    if(left_solid_neighbor)
    {
        lro=cro;
        lp=cp;
        lvx=-cvx;
    }

    if(right_solid_neighbor)
    {
        cro=lro;
        cp=lp;
        cvx=-lvx;
    }

    float3 rez = rieman_solver(lro,lp,lvx,cro,cp,cvx);

    rez.x=min(nsafe(rez.x),maxRo);
    rez.y=limiter(nsafe(rez.y),0.0f,rez.x*maxT);
    rez.z=nsafe(rez.z);
    data.R31[xp]=rez.x;
    data.G31[xp]=rez.y;
    data.B31[xp]=rez.z;
    if(data.shift>0||right_solid_neighbor||right_const_neighbor)
        data.P1[xp]=rez.x*rez.z;
}

__global__ void gpuGasSolver(GpuGasSolverData data)
{
    initPosition
    initPositionLR

    float lro, lp, le, lvx, lvy, lvs;
    float rro, rp, re, rvx, rvy, rvs;
    float ro, nro, p, np, vx, vy, vs, nvx, nvy, nvs, fl, fr;
    float dt=data.dt;

    ro=data.Ro0[xp];
    p=data.P0[xp];
    if(ro<-minRo||p<-minP)
    {
        return;
    }
    vx=data.Vx0[xp];
    vy=data.Vy0[xp];
    vs=data.Vs0[xp];

    lro=data.R31[xp];
    lp=data.G31[xp];
    lvx=data.B31[xp];

    if(xr!=xp)
    {
        rro=data.R31[xr];
        rp=data.G31[xr];
        rvx=data.B31[xr];
    }
    else
    {
        rro=data.Ro1[xp];
        rp=data.P1[xp];
        rvx=data.Vx1[xp];
    }

    lvy=lvx>0.0f?data.Vy1[xl]:data.Vy1[xp];
    rvy=rvx>0.0f?data.Vy1[xp]:data.Vy1[xr];
    lvs=lvx>0.0f?data.Vs1[xl]:data.Vs1[xp];
    rvs=rvx>0.0f?data.Vs1[xp]:data.Vs1[xr];

    fl=lro*lvx;
    fr=rro*rvx;

    nro=ro+(fl-fr)*dt;
    nro=nsafe(nro);

    if(nro<=minRo)
    {
        data.Ro0[xp]=0.0f;
        data.P0[xp]=0.0f;
        data.Vx0[xp]=0.0f;
        data.Vy0[xp]=0.0f;
        data.Ro1[xp]=ro;
        data.P1[xp]=fl;
        return;
    }

    nvx=(ro*vx+((fl*lvx-fr*rvx)+(lp-rp))*dt)/nro;
    nvy=(ro*vy+(fl*lvy-fr*rvy)*dt)/nro;
    nvs=(ro*vs+(fl*lvs-fr*rvs)*dt)/nro;

    float lv2=(lvx * lvx + lvy * lvy);
    float rv2=(rvx * rvx + rvy * rvy);
    le = lp / gm1 + 0.5f * lro * lv2;
    re = rp / gm1 + 0.5f * rro * rv2;

    np=(le*lv2*dt<lp*1e6f||re*rv2*dt<rp*1e6f)?
                ((p / gm1+((lp*lvx-rp*rvx)+(le*lvx-re*rvx))*dt + 0.5f *(ro * ((vx - nvx)*(vx + nvx) + (vy - nvy)*(vy + nvy))-(fl-fr)*dt*(nvx*nvx + nvy*nvy))) * gm1)
              :
                ((p +((lp*lvx-rp*rvx))*dt));

    p=min(1.0f,maxV/usafe(sqrtf(nvx*nvx+nvy*nvy)));
    nvx=nsafe(nvx)*p;
    nvy=nsafe(nvy)*p;
    nro=min(nro,maxRo);
    nvs=limiter(nsafe(nvs),0.0f,min(maxRoVisc,data.maxVisc)*nro*min(nro,1.0f));

    data.Ro0[xp]=nro;
    data.P0[xp]=limiter(nsafe(np),nro*minT,maxT*nro);

    data.Vx0[xp]=nvx;
    data.Vy0[xp]=nvy;
    data.Vs0[xp]=nvs;

    data.Ro1[xp]=ro;
    data.P1[xp]=fl;
}

__device__ float3 visc(float vx, float vx_rieman, float vxn,float vxnu,float vxnd,float vy,float vy_rieman,float vyn,float vynu,float vynd,float vs, float direction)
{
    float3 out;

    out.x=out.y=out.z=0.0f;

    if(vs>0.0f)
    {
        out.x = (vxn - vx) * 1.333333333f;// vx[i-1][j] - vx[i][j]
        out.x -= direction * (vynu - vynd) * 0.166666667f;// vy[i-1][j-1] - vy[i-1][j+1]
        out.z = vx_rieman * (vxn - vx) * 1.333333333f;// vx[i-0.5][j] * (vx[i-1][j] - vx[i][j])
        out.z -= direction * vxn * (vynu - vynd) * 0.166666667f;// vy[i-1][j] * (vy[i-1][j-1] - vy[i-1][j+1])

        out.y = (vyn - vy);// vy[i-1][j] - vy[i][j]
        out.y += direction * (vxnu - vxnd) * 0.25f;// vx[i-1][j-1] - vx[i-1][j+1]
        out.z += vy_rieman * (vyn - vy);// vy[i-0.5][j] * (vy[i-1][j] - vy[i][j])
        out.z += direction * vyn * (vxnu - vxnd) * 0.25f;// vx[i-1][j] * (vx[i-1][j-1] - vx[i-1][j+1])

        out.x = out.x * vs;
        out.y = out.y * vs;
        out.z = out.z * vs;
    }

    return out;
}

__global__ void gpuViscBuilder(GpuGasViscData data)
{
    initPositionXY

    int xlu = xl + xu - xp;
    int xru = xr + xu - xp;
    int xld = xl + xd - xp;
    int xrd = xr + xd - xp;

    float ro=data.Ro0[xp];
    float p=data.P0[xp];
    float vx=data.Vx0[xp];
    float vxn=data.Vx0[xl];
    float vxnu=data.Vx0[xlu];
    float vxnd=data.Vx0[xld];
    float vy=data.Vy0[xp];
    float vyn=data.Vy0[xl];
    float vynu=data.Vy0[xlu];
    float vynd=data.Vy0[xld];
    float vs=data.Vs0[xp];
    float vsn=data.Vs0[xl];
    float vsnu=data.Vs0[xlu];
    float vsnd=data.Vs0[xld];
    float vx_rieman = data.B31[xp];
    float vy_rieman = (vx_rieman>0.0f) ? data.Vy1[xl] : data.Vy1[xp];
    float maxVelosity;
    float dt=data.dt;
    if(ro<minRo||p<-minP)
    {
        data.R21[xp]=vx;
        data.G21[xp]=vy;
        data.B21[xp]=p;
        return;
    }

    vsn=min(min(vs,vsn),min(vsnu,vsnd));
    float3 rez = visc(vx, vx_rieman, vxn,vxnu,vxnd,vy,vy_rieman,vyn,vynu,vynd,vsn,1.0f);

    float vxvisc=rez.x;
    float vyvisc=rez.y;
    float evisc=rez.z;

    vxn=data.Vx0[xr];
    vxnu=data.Vx0[xru];
    vxnd=data.Vx0[xrd];
    vyn=data.Vy0[xr];
    vynu=data.Vy0[xru];
    vynd=data.Vy0[xrd];
    vsn=data.Vs0[xr];
    vsnu=data.Vs0[xru];
    vsnd=data.Vs0[xrd];

    if(xr!=xp)
    {
        vx_rieman=data.B31[xr];
    }
    else
    {
        vx_rieman=data.Vx1[xp];
    }

    vy_rieman=vx_rieman>0.0f?data.Vy1[xp]:data.Vy1[xr];

    vsn=min(min(vs,vsn),min(vsnu,vsnd));
    rez = visc(vx, vx_rieman, vxn,vxnu,vxnd,vy,vy_rieman,vyn,vynu,vynd,vsn,-1.0f);
    vxvisc+=rez.x;
    vyvisc+=rez.y;
    evisc+=rez.z;

    vxn=vx+vxvisc*dt/ro;
    vyn=vy+vyvisc*dt/ro;
    p=(p/gm1+0.5f*ro*((vx*vx-vxn*vxn)+(vy*vy-vyn*vyn))+evisc*dt)*gm1;

    maxVelosity=min(1.0f,maxV/usafe(sqrtf(vxn*vxn+vyn*vyn)));

    data.R21[xp]=nsafe(vxn)*maxVelosity;
    data.G21[xp]=nsafe(vyn)*maxVelosity;
    data.B21[xp]=limiter(nsafe(p),ro*minT,maxT*ro);
}

__global__ void gpuDataCorrect(GpuDataCorrectData data)
{
    initPosition
    initPositionLR

    float scheme=data.Sc1[xp];
    float Ro=data.Ro0[xp];
    float Rol=data.Ro0[xl];
    float Ror=data.Ro0[xr];
    float P=data.B21[xp];

    data.Vx0[xp]=data.R21[xp];
    data.Vy0[xp]=data.G21[xp];
    data.P0[xp]=P;

    if(scheme<=1.0f||min(Ro,min(Rol,Ror))<minRo||P<-minP)
    {
        if(Ro>=0.0f&&P>=0.0f)
        {
            data.Sc1[xp]=data.Sc0[xp];
            data.Cs1[xp]=data.Cs0[xp];
        }

        data.R11[xp]=data.R10[xp];
        data.R21[xp]=data.R20[xp];
        data.R31[xp]=data.R30[xp];
        data.G11[xp]=data.G10[xp];
        data.G21[xp]=data.G20[xp];
        data.G31[xp]=data.G30[xp];
        data.B11[xp]=data.B10[xp];
        data.B21[xp]=data.B20[xp];
        data.B31[xp]=data.B30[xp];
        return;
    }

    float La2=data.Cs1[xp];

    data.Sc1[xp]=correctG(data.Sc0[xl],data.Sc0[xp],data.Sc0[xr],La2,scheme);

    scheme=scheme+data.sc_param_correction;

    data.Cs1[xp]=correctG(data.Cs0[xl],data.Cs0[xp],data.Cs0[xr],La2,scheme);
    data.R11[xp]=correctG(data.R10[xl],data.R10[xp],data.R10[xr],La2,scheme);
    data.R21[xp]=correctG(data.R20[xl],data.R20[xp],data.R20[xr],La2,scheme);
    data.R31[xp]=correctG(data.R30[xl],data.R30[xp],data.R30[xr],La2,scheme);
    data.G11[xp]=correctG(data.G10[xl],data.G10[xp],data.G10[xr],La2,scheme);
    data.G21[xp]=correctG(data.G20[xl],data.G20[xp],data.G20[xr],La2,scheme);
    data.G31[xp]=correctG(data.G30[xl],data.G30[xp],data.G30[xr],La2,scheme);
    data.B11[xp]=correctG(data.B10[xl],data.B10[xp],data.B10[xr],La2,scheme);
    data.B21[xp]=correctG(data.B20[xl],data.B20[xp],data.B20[xr],La2,scheme);
    data.B31[xp]=correctG(data.B30[xl],data.B30[xp],data.B30[xr],La2,scheme);
}

#define solve(data,cdata,minVal,maxVal)\
val=cdata[xp];\
lval=((leftFluent<=0.0f)||left_solid_neighbor)?(val):(left_const_neighbor?data[xl]:cdata[xl]);\
rval=((rightFluent>=0.0f)||right_solid_neighbor)?(val):(right_const_neighbor?data[xr]:cdata[xr]);\
val=(data[xp]*ro+(leftFluent*lval-rightFluent*rval)*dt)/nro;\
data[xp]=limiter(nsafe(val),minVal,maxVal)

__global__ void gpuDataSolver(GpuDataSolverData data)
{
    initPosition
    initPositionLR

    float leftFluent, rightFluent, ro, p, nro,lval,val,rval,dt;
    ro=data.Ro1[xp];
    nro=data.Ro0[xp];
    p=data.P0[xp];

    data.Ro1[xp]=min(ro,p);
    if(ro<=-minRo||p<=-minP||nro<=-minRo)
        return;

    if(nro<minRo)
    {
        data.Sc0[xp]=0.0f;
        data.Cs0[xp]=0.0f;
        data.R10[xp]=0.0f;
        data.R20[xp]=0.0f;
        data.R30[xp]=0.0f;
        data.G10[xp]=0.0f;
        data.G20[xp]=0.0f;
        data.G30[xp]=0.0f;
        data.B10[xp]=0.0f;
        data.B20[xp]=0.0f;
        data.B30[xp]=0.0f;
        return;
    }

    leftFluent=data.P1[xp];

    if(xr!=xp)
    {
        rightFluent=data.P1[xr];
    }
    else
    {
        rightFluent=data.Vx1[xp]*ro;
    }

    dt=data.dt;

    bool left_solid_neighbor=data.Ro0[xl]<-minRo;
    bool left_const_neighbor=data.P0[xl]<-minP;
    bool right_solid_neighbor=data.Ro0[xr]<-minRo;
    bool right_const_neighbor=data.P0[xr]<-minP;

    solve(data.Sc0,data.Sc1,minSc,maxSc);
    solve(data.Cs0,data.Cs1,minCs,maxCs);
    solve(data.R10,data.R11,minColor,maxColor);
    solve(data.R20,data.R21,minColor,maxColor);
    solve(data.R30,data.R31,minColor,maxColor);
    solve(data.G10,data.G11,minColor,maxColor);
    solve(data.G20,data.G21,minColor,maxColor);
    solve(data.G30,data.G31,minColor,maxColor);
    solve(data.B10,data.B11,minColor,maxColor);
    solve(data.B20,data.B21,minColor,maxColor);
    solve(data.B30,data.B31,minColor,maxColor);
}

__global__ void gpuGravity(GpuGravityData data)
{
    initPosition;
    float x = (float)(xp % width);
    float y = (float)(data.stringShift + xp / width);

    float ax=0.0f;
    float ay=0.0f;

    float dx, dy, dist, ndist, rad, G, bx, by;

    for(int i=0;i<data.bodyCount;i++)
    {
        bx=bodys[i].x;
        by=bodys[i].y;
        if(bx>=x&&by>=y&&bx<x+1.0f&&by<y+1.0f)
        {
            data.bRo[i]=data.Ro0[xp];
            data.bVx[i]=data.Vx0[xp];
            data.bVy[i]=data.Vy0[xp];
        }
        else
        {
            dx=bx-x-0.5f;
            dy=by-y-0.5f;
            dist=sqrtf(dx*dx+dy*dy);
            rad=max(bodys[i].radius,0.5f);
            ndist=dist/rad;
            G=(ndist>=1.0f?1.0f/dist:((3.0f*ndist-2.0f*ndist*ndist)/rad))*bodys[i].mass;
            ax+=G*dx/dist;
            ay+=G*dy/dist;
        }
    }

    dx=dx+ax*data.dt;
    dy=dy+ay*data.dt;

    dist=min(1.0f,maxV/usafe(sqrtf(dx*dx+dy*dy)));

    data.Vx0[xp]=nsafe(dx)*dist;
    data.Vy0[xp]=nsafe(dy)*dist;
}

__global__ void gpuBodyGravity(GpuBodyGravityData data)
{
    int xp = blockIdx.x * blockDim.x + threadIdx.x;
    int total = data.bodyCount;
    float x=bodys[xp].x;
    float y=bodys[xp].y;

    if(xp>=total||y<data.fromString||y>=data.toString)
    {
        return;
    }

    float ax=0.0f;
    float ay=0.0f;

    float dx, dy, dist, ndist, rad, G, bx, by;

    for(int i=0;i<total; ++i)
    {
        bx=bodys[i].x;
        by=bodys[i].y;
        dx=bx-x;
        dy=by-y;
        dist=sqrtf(dx*dx+dy*dy);

        if(dist<1e-3f)
            continue;

        rad=max(bodys[i].radius,0.5f);
        ndist=dist/rad;
        G=(ndist>=1.0f?1.0f/dist:((3.0f*ndist-2.0f*ndist*ndist)/rad))*bodys[i].mass;
        ax+=G*dx/dist;
        ay+=G*dy/dist;
    }

    data.bAx[xp]=ax*data.dt;
    data.bAy[xp]=ay*data.dt;
}

__global__ void gpuFinalize(GpuFinalizeData data)
{
    initPositionXY

    float ro=data.Ro0[xp];
    float p=data.P0[xp];
    float vx=data.Vx0[xp];
    float vy=data.Vy0[xp];
    float vs=data.Vs0[xp];
    float v=sqrtf(vx*vx+vy*vy);

    //dt
    float ndt=1e3f;
    if(p>0.0f&&ro>minRo)
    {
        ndt=min(1e3f,min(0.75f/usafe(sqrtf(p*gm/usafe(ro))+v),0.375f*(max(minRo,ro)/usafe(vs))));
    }
    data.Vs1[xp]=ndt;

    //hardstep
    if(p<=-minP||ro<=-minP)
    {
        float fullTimeLeft=data.Cs1[xp];
        float markTimeLeft=data.Sc1[xp];
        float dt=data.dt;

        if(fullTimeLeft>0.0f)
        {
            fullTimeLeft-=dt;
            data.Cs1[xp]=fullTimeLeft;
        }

        vs=min(-minRo,-minP);
        float neibors=((data.Ro1[xl]<vs)?1.0f:0.0f)+
                    ((data.Ro1[xr]<vs)?1.0f:0.0f)+
                    ((data.Ro1[xu]<vs)?1.0f:0.0f)+
                    ((data.Ro1[xd]<vs)?1.0f:0.0f);

        if(markTimeLeft>0.0f && neibors<3.5f)
        {
            markTimeLeft-=dt*sqrtf(4.0f-neibors);
            data.Sc1[xp]=markTimeLeft;
        }

        if(fullTimeLeft<=0.0f||markTimeLeft<=0.0f)
        {
            data.Ro0[xp]=fabs(ro);
            data.P0[xp]=fabs(p);
        }
    }
}

__global__ void gpuDtIteration(GpuDtData data)
{
    __shared__ float cache[gpuMaxBlockSize];
    int x=blockIdx.x * blockDim.x + threadIdx.x + data.width;
    int scale=blockDim.x;
    cache[threadIdx.x]=data.data[x];
    while(scale>1)
    {
        scale=scale>>1;
        __syncthreads();
        if(threadIdx.x<scale)
        {
            cache[threadIdx.x]=min(cache[threadIdx.x],cache[threadIdx.x+scale]);
        }
    }
    if(threadIdx.x==0)
    {
        data.data[x]=cache[threadIdx.x];
    }
}

__global__ void gpuDtCopy(GpuDtData data)
{
    if(threadIdx.x==0)
    {
        data.data[blockIdx.x + data.width]=data.data[blockIdx.x * blockDim.x + data.width];
    }
}

__global__ void gpuRender(GpuRenderData data)
{
    initPosition

    float ro=data.Ro0[xp];
    float p=data.P0[xp];
    float vx=data.Vx0[xp];
    float vy=data.Vy0[xp];
    float v=sqrtf(vx*vx+vy*vy);

    float cs=data.Cs0[xp];

    float cr1=data.R10[xp];
    float cr2=data.R20[xp];
    float cr3=data.R30[xp];
    float cg1=data.G10[xp];
    float cg2=data.G20[xp];
    float cg3=data.G30[xp];
    float cb1=data.B10[xp];
    float cb2=data.B20[xp];
    float cb3=data.B30[xp];

    float c3=usafe(fabs(ro)+fabs(p)+v);
    float c1=powf(fabs(ro)/c3,cs);
    float c2=powf(fabs(p)/c3,cs);
          c3=powf(v/c3,cs);

    float cc3=usafe(c1+c2+c3);
    float cc1=c1/cc3;
    float cc2=c2/cc3;
          cc3=c3/cc3;

    c1=max(0.0f,(1.0f-fabs(ro)*1e3f));
    c1=max(1.0f,1.0f-c1*c1);

    data.R11[xp]=min(256.0f,max(0.0f,(cr1*cc1+cr2*cc2+cr3*cc3)*c1));
    data.G11[xp]=min(256.0f,max(0.0f,(cg1*cc1+cg2*cc2+cg3*cc3)*c1));
    data.B11[xp]=min(256.0f,max(0.0f,(cb1*cc1+cb2*cc2+cb3*cc3)*c1));
}


__global__ void gpuCopyStrings(GpuCopyStringData data)
{
    int from=gridDim.x * blockDim.x * data.srcString + blockIdx.x * blockDim.x + threadIdx.x;
    int tost=gridDim.x * blockDim.x * data.dstString + blockIdx.x * blockDim.x + threadIdx.x;

    data.data0[tost]=data.data0[from];
    data.data1[tost]=data.data1[from];
    data.data2[tost]=data.data2[from];
    if(data.count==3)
        return;
    data.data3[tost]=data.data3[from];
    data.data4[tost]=data.data4[from];
    if(data.count==5)
        return;
    data.data5[tost]=data.data5[from];
    data.data6[tost]=data.data6[from];
    data.data7[tost]=data.data7[from];
    data.data8[tost]=data.data8[from];
    data.data9[tost]=data.data9[from];
    data.data10[tost]=data.data10[from];
    data.data11[tost]=data.data11[from];
    if(data.count==12)
        return;
    data.data12[tost]=data.data12[from];
    data.data13[tost]=data.data13[from];
    data.data14[tost]=data.data14[from];
    data.data15[tost]=data.data15[from];
}

#define CUDA_ERROR_HANDLE(x,comment)\
cudaError_t error=x;\
if(error!=cudaSuccess)\
    std::cout<<comment<<std::string(" cuda error : ") << cudaGetErrorString(error)<<std::endl

#define CUDA_BLOCK_OPTIMAZER(functon,blockSize,where)\
cudaError_t error;\
do\
{\
    functon<<<dim3(size/blockSize),dim3(blockSize)>>>(data);\
    error= cudaGetLastError();\
    if(error!=cudaSuccess)\
    {\
        blockSize/=2;\
    }\
} while(error!=cudaSuccess && blockSize>=1);\
if(blockSize<1||error!= cudaSuccess)\
{\
    std::cout<<"in "<<where<<" cuda error : "<<cudaGetErrorString(error)<<std::endl;\
    throw std::string(cudaGetErrorString(error));\
}

int cuda_gas_correct_block_size=gpuMaxBlockSize;

void cudaGasCorrect(GpuGasCorrectData data, int size)
{
    CUDA_BLOCK_OPTIMAZER(gpuGasCorrect,cuda_gas_correct_block_size,"cudaGasCorrect")
}

int cuda_gas_rieman_block_size=gpuMaxBlockSize;

void cudaGasRieman(GpuGasRiemanData data, int size)
{
    if(data.shift)
    {
        int block=gpuMaxBlockSide;
        CUDA_BLOCK_OPTIMAZER(gpuGasRieman,block,"cudaGasRieman")
    }
    else
    {
        CUDA_BLOCK_OPTIMAZER(gpuGasRieman,cuda_gas_rieman_block_size,"cudaGasRieman")
    }
}

int cuda_visc_builder_block_size=gpuMaxBlockSize;

void cudaViscBuilder(GpuGasViscData data, int size)
{
    CUDA_BLOCK_OPTIMAZER(gpuViscBuilder,cuda_visc_builder_block_size,"cudaViscBuilder")
}

int cuda_gas_solver_block_size=gpuMaxBlockSize;

void cudaGasSolver(GpuGasSolverData data, int size)
{
    CUDA_BLOCK_OPTIMAZER(gpuGasSolver,cuda_gas_solver_block_size,"cudaGasSolver")
}

int cuda_data_correct_block_size=gpuMaxBlockSize;

void cudaDataCorrect(GpuDataCorrectData data, int size)
{
    CUDA_BLOCK_OPTIMAZER(gpuDataCorrect,cuda_data_correct_block_size,"cudaDataCorrect")
}

int cuda_data_solver_block_size=gpuMaxBlockSize;

void cudaDataSolver(GpuDataSolverData data, int size)
{
    CUDA_BLOCK_OPTIMAZER(gpuDataSolver,cuda_data_solver_block_size,"cudaDataSolver")
}

int cuda_gravity_block_size=gpuMaxBlockSize;

void cudaGravity(GpuGravityData data, int size)
{
    CUDA_BLOCK_OPTIMAZER(gpuGravity,cuda_gravity_block_size,"cudaGravity")
}

int cuda_body_gravity_block_size=gpuMaxBlockSize;

void cudaBodyGravity(GpuBodyGravityData data, int size)
{
    CUDA_BLOCK_OPTIMAZER(gpuBodyGravity,cuda_gravity_block_size,"cudaBodyGravity")
}

int cuda_finalize_block_size=gpuMaxBlockSize;

void cudaFinalize(GpuFinalizeData data, int size)
{
    CUDA_BLOCK_OPTIMAZER(gpuFinalize,cuda_finalize_block_size,"cudaFinalize")
}

int cuda_dt_block_size=gpuMaxBlockSize;

int cudaDtIteration(GpuDtData data, int size)
{
    size=size>cuda_dt_block_size?size:cuda_dt_block_size;
    CUDA_BLOCK_OPTIMAZER(gpuDtIteration,cuda_dt_block_size,"cudaDtIteration");

    return cuda_dt_block_size;
}

void cudaDtCopy(GpuDtData data, int size)
{
    size=size>cuda_dt_block_size?size:cuda_dt_block_size;
    CUDA_BLOCK_OPTIMAZER(gpuDtCopy,cuda_dt_block_size,"cudaDtCopy");
}

int cuda_render_block_size=gpuMaxBlockSize;

void cudaRender(GpuRenderData data, int size)
{
    CUDA_BLOCK_OPTIMAZER(gpuRender,cuda_render_block_size,"cudaRender");
}

int cuda_copy_string_block_size=gpuMaxBlockSide;

void cudaCopyStrings(GpuCopyStringData data, int size)
{
    gpuCopyStrings<<<dim3(size/cuda_copy_string_block_size),dim3(cuda_copy_string_block_size)>>>(data);
    cudaError_t error= cudaGetLastError();
    if(error!= cudaSuccess)
    {
        std::cout<<"in cudaCopyStrings cuda error : "<<cudaGetErrorString(error)<<std::endl;
        throw std::string(cudaGetErrorString(error));
    }
}

GpuInfo cudaGetGpuInfo(int gpuId)
{
    GpuInfo gpuInfo;
    gpuInfo.gpuId=gpuId;

    cudaDeviceProp info;

    CUDA_ERROR_HANDLE(cudaGetDeviceProperties(&info,gpuId),"in cudaGetGpuInfo");
    if(error!=cudaSuccess)
        throw "getting device properties error";

    if(info.maxThreadsPerBlock<gpuMaxBlockSize)
    {
        cuda_data_correct_block_size=
        cuda_data_solver_block_size=
        cuda_dt_block_size=
        cuda_gravity_block_size=
        cuda_body_gravity_block_size=
        cuda_finalize_block_size=
        cuda_gas_correct_block_size=
        cuda_gas_rieman_block_size=
        cuda_gas_solver_block_size=
        cuda_visc_builder_block_size=
        cuda_render_block_size=info.maxThreadsPerBlock;
    }
    gpuInfo.memory=info.totalGlobalMem*MEMORY_EFFECTIVE_MULTIPLICATOR/BYTES_PER_PIXEL;

    return gpuInfo;
}

bool cudaSelectGpu(int gpuId)
{
    CUDA_ERROR_HANDLE(cudaSetDevice(gpuId),"in cudaSelectGpu");

    if(error!=cudaSuccess)
        throw std::string("can't select gpu");
        return true;
}

float cudaGetMinDt(float * data, float * cpuBufer, int width)
{
    CUDA_ERROR_HANDLE(cudaMemcpy((void *)cpuBufer,data,(width+32)*sizeof(float),cudaMemcpyDeviceToHost),"in cudaGetMinDt");
    if(error!=cudaSuccess)
        throw std::string("get min dt error");
    return cpuBufer[width];
}

void cudaSetConstData(GpuGravityBody * data)
{
    CUDA_ERROR_HANDLE(cudaMemcpyToSymbol(bodys,data,(4096)*sizeof(GpuGravityBody)),"in cudaSetConstData");
    if(error!=cudaSuccess)
        throw std::string("get min dt error");
}

float * cudaAllocateMemory(int size)
{
    float * p;
    CUDA_ERROR_HANDLE(cudaMalloc((void**)&p,size*sizeof(float)),"in cudaAllocateMemory");
    if(error!=cudaSuccess)
        throw std::string("allocation error");
    return p;
}

bool cudaFreeMemory(float * p)
{
    CUDA_ERROR_HANDLE(cudaFree(p),"in cudaFreeMemory");
    if(error!=cudaSuccess)
    {
        throw std::string("can't free memory");
    }
    return true;
}

float * cudaAllocateHostMemory(int size)
{
    float * out;
    CUDA_ERROR_HANDLE(cudaHostAlloc((void **)&out,size*sizeof(float),cudaHostAllocPortable),"in cudaAllocateHostMemory");
    if(error!=cudaSuccess)
        throw std::string("allocation host error");
    return out;
}

GpuGravityBody * cudaAllocateHostMemoryBody(int size)
{
    GpuGravityBody * out;
    CUDA_ERROR_HANDLE(cudaHostAlloc((void **)&out,size*sizeof(GpuGravityBody),cudaHostAllocPortable),"in cudaAllocateHostMemoryBody");
    if(error!=cudaSuccess)
        throw std::string("allocation host body error");
    return out;
}

bool cudaFreeHostMemory(float * ptr)
{
    CUDA_ERROR_HANDLE(cudaFreeHost(ptr),"in cudaFreeHostMemory");
    if(error!=cudaSuccess)
        throw std::string("can't free host memory");

    return true;
}

bool cudaFreeHostMemory(GpuGravityBody * ptr)
{
    CUDA_ERROR_HANDLE(cudaFreeHost(ptr),"in cudaFreeHostMemory");
    if(error!=cudaSuccess)
        throw std::string("can't free host body memory");

    return true;
}

bool cudaPutToGpu(float * gpu,float * cpu, int size)
{
    CUDA_ERROR_HANDLE(cudaMemcpy(gpu,cpu,size*sizeof(float),cudaMemcpyHostToDevice),"in cudaPutToGpu");
    if(error!=cudaSuccess)
    {
        throw std::string("can't put to gpu");
    }
    return true;
}

bool cudaGetFromGpu(float * cpu, float * gpu, int size)
{
    CUDA_ERROR_HANDLE(cudaMemcpy(cpu,gpu,size*sizeof(float),cudaMemcpyDeviceToHost),"in cudaGetFromGpu");
    if(error!=cudaSuccess)
    {
        throw std::string("can't get from gpu");
    }
    return true;
}

std::vector<std::string> cudaGetGpusInfo()
{
    std::vector<std::string> out;

    char buf[256];
    int deviceCount;

    {
    CUDA_ERROR_HANDLE(cudaGetDeviceCount(&deviceCount),"in cudaGetGpusInfo");
    }

    sprintf( buf, "%d", deviceCount );
    out.push_back(std::string(""));
    out.push_back(std::string("total gpus : ") + buf);
    out.push_back(std::string(""));

    for(int i=0;i<deviceCount;i++)
    {
        cudaDeviceProp info;
        CUDA_ERROR_HANDLE(cudaGetDeviceProperties(&info,i),"in cudaGetGpusInfo");
        sprintf( buf, "%d", i);
        out.push_back(std::string("gpu # ") + buf);
        out.push_back(std::string("name : ") + info.name);
        sprintf( buf, "%d", info.multiProcessorCount);
        out.push_back(std::string("multiprocessor count : ") + buf);
        sprintf( buf, "%d", info.clockRate/1000);
        out.push_back(std::string("processor speed : ") + buf + " mgz");
        sprintf( buf, "%d", int(info.totalGlobalMem/(1024*1024)));
        out.push_back(std::string("total memory : ") + buf + " mb");
        sprintf( buf, "%d", info.memoryBusWidth);
        out.push_back(std::string("memory bus width : ") + buf + " bit");
        sprintf( buf, "%d", info.memoryClockRate/1000);
        out.push_back(std::string("memory speed : ") + buf + " mgz");
        sprintf( buf, "%d", (int)info.totalConstMem/1000);
        out.push_back(std::string("total const memory : ") + buf + " kb");
        sprintf( buf, "%d", info.maxThreadsPerBlock);
        out.push_back(std::string("max threads per block : ") + buf);
        sprintf( buf, "%d", int(info.sharedMemPerBlock));
        out.push_back(std::string("shared memory per block : ") + buf + " bytes");
        sprintf( buf, "%d", info.regsPerBlock);
        out.push_back(std::string("registers per block : ") + buf + " bytes");
        sprintf( buf, "%d", info.maxGridSize[0]);
        out.push_back(std::string("max grid size x block : ") + buf);
        sprintf( buf, "%d", info.maxThreadsDim[0]);
        out.push_back(std::string("max thread size x block : ") + buf);

        out.push_back(std::string(""));
        out.push_back(std::string(""));
    }

    return out;
}
