#include "gpudata.h"

const char * GpuInfo::GPU_MEMORY="gpu_memory";
const char * GpuInfo::GPU_ID="gpu_id";

GpuGasCorrectData GpuData::getGpuGasCorrectData(bool transponate, int width, float dt)
{
    GpuGasCorrectData out;

    out.transponate=transponate;
    out.width=width;
    out.dt=dt;

    out.Ro0=Ro0;
    out.Ro1=Ro1;
    out.P0=P0;
    out.P1=P1;
    out.Vx0=Vx0;
    out.Vx1=Vx1;
    out.Vy0=Vy0;
    out.Vy1=Vy1;
    out.Vs0=Vs0;
    out.Vs1=Vs1;
    out.Sc0=Sc0;
    out.Sc1=Sc1;
//    out.Cs0=Cs0;
    out.Cs1=Cs1;
//    out.R10=R10;
//    out.R11=R11;
//    out.G10=G10;
//    out.G11=G11;
//    out.B10=B10;
//    out.B11=B11;
//    out.R20=R20;
//    out.R21=R21;
//    out.G20=G20;
//    out.G21=G21;
//    out.B20=B20;
//    out.B21=B21;
//    out.R30=R30;
//    out.R31=R31;
//    out.G30=G30;
//    out.G31=G31;
//    out.B30=B30;
//    out.B31=B31;

    if(transponate)
    {
        out.Vx0 = Vy0;
        out.Vy0 = Vx0;
        out.Vx1 = Vy1;
        out.Vy1 = Vx1;
    }

    return out;
}

GpuGasRiemanData GpuData::getGpuGasRiemanData(bool transponate, int width)
{
    GpuGasRiemanData out;

    out.transponate=transponate;
    out.width=width;
    out.shift=0;

    out.Ro0=Ro0;
    out.Ro1=Ro1;
    out.P0=P0;
    out.P1=P1;
    out.Vx0=Vx0;
    out.Vx1=Vx1;
//    out.Vy0=Vy0;
//    out.Vy1=Vy1;
//    out.Vs0=Vs0;
//    out.Vs1=Vs1;
//    out.Sc0=Sc0;
//    out.Sc1=Sc1;
//    out.Cs0=Cs0;
//    out.Cs1=Cs1;
//    out.R10=R10;
//    out.R11=R11;
//    out.G10=G10;
//    out.G11=G11;
//    out.B10=B10;
//    out.B11=B11;
//    out.R20=R20;
//    out.R21=R21;
//    out.G20=G20;
//    out.G21=G21;
//    out.B20=B20;
//    out.B21=B21;
//    out.R30=R30;
    out.R31=R31;
//    out.G30=G30;
    out.G31=G31;
//    out.B30=B30;
    out.B31=B31;

    if(transponate)
    {
        out.Vx0 = Vy0;
        out.Vx1 = Vy1;
    }

    return out;
}

GpuGasSolverData GpuData::getGpuGasSolverData(bool transponate, int width, float dt, float maxVisc)
{
    GpuGasSolverData out;

    out.transponate=transponate;
    out.width=width;
    out.dt=dt;
    out.maxVisc=maxVisc;

    out.Ro0=Ro0;
    out.Ro1=Ro1;
    out.P0=P0;
    out.P1=P1;
    out.Vx0=Vx0;
    out.Vx1=Vx1;
    out.Vy0=Vy0;
    out.Vy1=Vy1;
    out.Vs0=Vs0;
    out.Vs1=Vs1;
//    out.Sc0=Sc0;
//    out.Sc1=Sc1;
//    out.Cs0=Cs0;
//    out.Cs1=Cs1;
//    out.R10=R10;
//    out.R11=R11;
//    out.G10=G10;
//    out.G11=G11;
//    out.B10=B10;
//    out.B11=B11;
//    out.R20=R20;
//    out.R21=R21;
//    out.G20=G20;
//    out.G21=G21;
//    out.B20=B20;
//    out.B21=B21;
//    out.R30=R30;
    out.R31=R31;
//    out.G30=G30;
    out.G31=G31;
//    out.B30=B30;
    out.B31=B31;
    if(transponate)
    {
        out.Vx0 = Vy0;
        out.Vy0 = Vx0;
        out.Vx1 = Vy1;
        out.Vy1 = Vx1;
    }

    return out;
}


GpuGasViscData GpuData::getGpuGasViscData(bool transponate, int width, float dt)
{
    GpuGasViscData out;

    out.transponate=transponate;
    out.width=width;
    out.dt=dt;

    out.Ro0=Ro0;
//    out.Ro1=Ro1;
    out.P0=P0;
//    out.P1=P1;
    out.Vx0=Vx0;
    out.Vx1=Vx1;
    out.Vy0=Vy0;
    out.Vy1=Vy1;
    out.Vs0=Vs0;
//    out.Vs1=Vs1;
//    out.Sc0=Sc0;
//    out.Sc1=Sc1;
//    out.Cs0=Cs0;
//    out.Cs1=Cs1;
//    out.R10=R10;
//    out.R11=R11;
//    out.G10=G10;
//    out.G11=G11;
//    out.B10=B10;
//    out.B11=B11;
//    out.R20=R20;
    out.R21=R21;
//    out.G20=G20;
    out.G21=G21;
//    out.B20=B20;
    out.B21=B21;
//    out.R30=R30;
//    out.R31=R31;
//    out.G30=G30;
//    out.G31=G31;
//    out.B30=B30;
    out.B31=B31;

    if(transponate)
    {
        out.Vx0 = Vy0;
        out.Vy0 = Vx0;
        out.Vx1 = Vy1;
        out.Vy1 = Vx1;
        out.R21=G21;
        out.G21=R21;
    }

    return out;
}

GpuDataCorrectData GpuData::getGpuDataCorrectData(bool transponate, int width, float sc_param_correction)
{
    GpuDataCorrectData out;

    out.transponate=transponate;
    out.width=width;
    out.sc_param_correction=sc_param_correction;

    out.Ro0=Ro0;
//    out.Ro1=Ro1;
    out.P0=P0;
//    out.P1=P1;
    out.Vx0=Vx0;
//    out.Vx1=Vx1;
    out.Vy0=Vy0;
//    out.Vy1=Vy1;
//    out.Vs0=Vs0;
//    out.Vs1=Vs1;
    out.Sc0=Sc0;
    out.Sc1=Sc1;
    out.Cs0=Cs0;
    out.Cs1=Cs1;
    out.R10=R10;
    out.R11=R11;
    out.G10=G10;
    out.G11=G11;
    out.B10=B10;
    out.B11=B11;
    out.R20=R20;
    out.R21=R21;
    out.G20=G20;
    out.G21=G21;
    out.B20=B20;
    out.B21=B21;
    out.R30=R30;
    out.R31=R31;
    out.G30=G30;
    out.G31=G31;
    out.B30=B30;
    out.B31=B31;

    return out;
}

GpuDataSolverData GpuData::getGpuDataSolverData(bool transponate, int width, float dt)
{
    GpuDataSolverData out;

    out.transponate=transponate;
    out.width=width;
    out.dt=dt;

    out.Ro0=Ro0;
    out.Ro1=Ro1;
    out.P0=P0;
    out.P1=P1;
//    out.Vx0=Vx0;
    out.Vx1=Vx1;
//    out.Vy0=Vy0;
//    out.Vy1=Vy1;
//    out.Vs0=Vs0;
//    out.Vs1=Vs1;
    out.Sc0=Sc0;
    out.Sc1=Sc1;
    out.Cs0=Cs0;
    out.Cs1=Cs1;
    out.R10=R10;
    out.R11=R11;
    out.G10=G10;
    out.G11=G11;
    out.B10=B10;
    out.B11=B11;
    out.R20=R20;
    out.R21=R21;
    out.G20=G20;
    out.G21=G21;
    out.B20=B20;
    out.B21=B21;
    out.R30=R30;
    out.R31=R31;
    out.G30=G30;
    out.G31=G31;
    out.B30=B30;
    out.B31=B31;

    if(transponate)
    {
        out.Vx1 = Vy1;
    }

    return out;
}

GpuGravityData GpuData::getGpuGravityData(int stringShift, int bodyCount, int width, float dt)
{
    GpuGravityData out;

    out.Ro0=Ro0;
//    out.Ro1=Ro1;
//    out.P0=P0;
//    out.P1=P1;
    out.Vx0=Vx0;
//    out.Vx1=Vx1;
    out.Vy0=Vy0;
//    out.Vy1=Vy1;
//    out.Vs0=Vs0;
//    out.Vs1=Vs1;
//    out.Sc0=Sc0;
//    out.Sc1=Sc1;
//    out.Cs0=Cs0;
//    out.Cs1=Cs1;
//    out.R10=R10;
//    out.R11=R11;
//    out.G10=G10;
//    out.G11=G11;
//    out.B10=B10;
//    out.B11=B11;
//    out.R20=R20;
//    out.R21=R21;
//    out.G20=G20;
//    out.G21=G21;
//    out.B20=B20;
//    out.B21=B21;
//    out.R30=R30;
//    out.R31=R31;
//    out.G30=G30;
//    out.G31=G31;
//    out.B30=B30;
//    out.B31=B31;

    return out;
}

GpuBodyGravityData GpuData::getGpuBodyGravityData(int fromString, int toString, int bodyCount, float dt)
{
    GpuBodyGravityData out;
    out.bAx=bAx;
    out.bAy=bAy;
    out.dt=dt;
    out.fromString=fromString;
    out.toString=toString;
    out.bodyCount=bodyCount;
    return out;
}

GpuFinalizeData GpuData::getGpuFinalizeData(bool transponate, int width, float dt)
{
    GpuFinalizeData out;

    out.transponate=transponate;
    out.width=width;
    out.dt=dt;

    out.Ro0=Ro0;
    out.Ro1=Ro1;
    out.P0=P0;
//    out.P1=P1;
    out.Vx0=Vx0;
//    out.Vx1=Vx1;
    out.Vy0=Vy0;
//    out.Vy1=Vy1;
    out.Vs0=Vs0;
    out.Vs1=Vs1;
//    out.Sc0=Sc0;
    out.Sc1=Sc1;
//    out.Cs0=Cs0;
    out.Cs1=Cs1;
//    out.R10=R10;
//    out.R11=R11;
//    out.G10=G10;
//    out.G11=G11;
//    out.B10=B10;
//    out.B11=B11;
//    out.R20=R20;
//    out.R21=R21;
//    out.G20=G20;
//    out.G21=G21;
//    out.B20=B20;
//    out.B21=B21;
//    out.R30=R30;
//    out.R31=R31;
//    out.G30=G30;
//    out.G31=G31;
//    out.B30=B30;
//    out.B31=B31;

    if(transponate)
    {
        out.Vx0 = Vy0;
        out.Vy0 = Vx0;
    }

    return out;
}

GpuRenderData GpuData::getGpuRenderData(bool transponate, int width)
{
    GpuRenderData out;

    out.width=width;

    out.Ro0=Ro0;
//    out.Ro1=Ro1;
    out.P0=P0;
//    out.P1=P1;
    out.Vx0=Vx0;
//    out.Vx1=Vx1;
    out.Vy0=Vy0;
//    out.Vy1=Vy1;
//    out.Vs0=Vs0;
//    out.Vs1=Vs1;
//    out.Sc0=Sc0;
//    out.Sc1=Sc1;
    out.Cs0=Cs0;
//    out.Cs1=Cs1;
    out.R10=R10;
    out.R11=R11;
    out.G10=G10;
    out.G11=G11;
    out.B10=B10;
    out.B11=B11;
    out.R20=R20;
//    out.R21=R21;
    out.G20=G20;
//    out.G21=G21;
    out.B20=B20;
//    out.B21=B21;
    out.R30=R30;
//    out.R31=R31;
    out.G30=G30;
//    out.G31=G31;
    out.B30=B30;
//    out.B31=B31;

    if(transponate)
    {
        out.Vx0 = Vy0;
        out.Vy0 = Vx0;
    }

    return out;
}

GpuCopyStringData GpuData::getGpuCopyStringDataBase(int dstString, int srcString)
{
    GpuCopyStringData out;

    out.dstString=dstString;
    out.srcString=srcString;

    out.data0=Ro0;
    out.data1=P0;
    out.data2=Vx0;
    out.data3=Vy0;
    out.data4=Vs0;
    out.data5=Sc0;
    out.data6=Cs0;
    out.data7=R10;
    out.data8=R20;
    out.data9=R30;
    out.data10=G10;
    out.data11=G20;
    out.data12=G30;
    out.data13=B10;
    out.data14=B20;
    out.data15=B30;

    out.count=16;

    return out;
}

GpuCopyStringData GpuData::getGpuCopyStringDataCorrectGas( int dstString, int srcString)
{
    GpuCopyStringData out;

    out.dstString=dstString;
    out.srcString=srcString;

    out.data0=Ro1;
    out.data1=P1;
    out.data2=Vx1;
    out.data3=Vy1;
    out.data4=Vs1;

    out.count=5;

    return out;
}

GpuCopyStringData GpuData::getGpuCopyStringDataSolvedGas(int dstString, int srcString)
{
    GpuCopyStringData out;

    out.dstString=dstString;
    out.srcString=srcString;

    out.data0=Vs0;
    out.data1=Vx0;
    out.data2=Vy0;

    out.count=3;

    return out;
}

GpuCopyStringData GpuData::getGpuCopyStringDataCorrectData(int dstString, int srcString)
{
    GpuCopyStringData out;

    out.dstString=dstString;
    out.srcString=srcString;

    out.data0=Vs1;
    out.data1=Sc1;
    out.data2=Cs1;
    out.data3=R11;
    out.data4=R21;
    out.data5=R31;
    out.data6=G11;
    out.data7=G21;
    out.data8=G31;
    out.data9=B11;
    out.data10=B21;
    out.data11=B31;

    out.count=12;

    return out;
}

GpuDtData GpuData::getGpuDtData(int width)
{
    GpuDtData out;
    out.width=width;
    out.data=Vs1;
    return out;
}
