#ifndef GPUDATA_H
#define GPUDATA_H

struct GpuInfo
{
    static const char * GPU_MEMORY;
    int memory;
    static const char * GPU_ID;
    int gpuId;
    GpuInfo():gpuId(0),memory(0){}
};

//формат данных

//верхняя граница [-1]
//строка 1 [0]
//-------- []
//строка n [n-1]
//нижняя граница [n]


struct GpuGasCorrectData
{
    float * Ro0, * Ro1; //cro//   //   //   //
    float * P0, * P1;   //c p//   //   //   //
    float * Vx0, * Vx1; //cvx//   //   //   //
    float * Vy0, * Vy1; //cvy//   //   //   //
    float * Vs0, * Vs1; //la2//   //   //cvs//solve
    float * Sc0, * Sc1; //   //   //   //csc//solve
    float        * Cs1;
    float dt; // шаг по времени
    int width;
    bool transponate;
};

struct GpuGasRiemanData
{
    float * Ro0, * Ro1; //cro//   //   //   //
    float * P0, * P1;   //c p//   //   //   //
    float * Vx0, * Vx1; //cvx//   //   //   //
    float * R31; //   //rro//   //cr3//solve
    float * G31; //   //r p//   //cg3//solve
    float * B31; //   //rvx//   //cb3//solve
    int width;
    int shift;
    bool transponate;
};

struct GpuGasSolverData
{
    float * Ro0, * Ro1; //cro//   //   //   //
    float * P0, * P1;   //c p//   //   //   //
    float * Vx0, * Vx1; //cvx//   //   //   //
    float * Vy0, * Vy1; //cvy//   //   //   //
    float * Vs0, * Vs1; //cvy//   //   //   //
    float * R31; //   //rro//   //cr3//solve
    float * G31; //   //r p//   //cg3//solve
    float * B31; //   //rvx//   //cb3//solve
    float dt; // шаг по времени
    int width;
    bool transponate;
    float maxVisc;
};

struct GpuGasViscData
{
    float * Ro0;
    float * Vx0, * Vx1;
    float * Vy0, * Vy1;
    float * P0;
    float * Vs0;
    float * B31;
    float * R21;
    float * G21;
    float * B21;
    float dt;
    int width;
    bool transponate;
};

struct GpuDataCorrectData
{
    float * Ro0; //cro//   //   //   //
    float * P0;   //c p//   //   //   //
    float * Vx0; //cvx//   //   //   //
    float * Vy0; //cvy//   //   //   //
    float * Sc0, * Sc1; //   //   //   //csc//solve
    float * Cs0, * Cs1; //   //   //   //ccs//solve
    float * R10, * R11; //   //   // ro//cr1//solve
    float * G10, * G11; //   //   // fl//cg1//solve
    float * B10, * B11; //   //   //   //cb1//solve
    float * R20, * R21; //   //   //   //cr2//solve
    float * G20, * G21; //   //   //   //cg2//solve
    float * B20, * B21; //   //   //   //cb2//solve
    float * R30, * R31; //   //rro//   //cr3//solve
    float * G30, * G31; //   //r p//   //cg3//solve
    float * B30, * B31; //   //rvx//   //cb3//solve
    int width;
    bool transponate;
    float sc_param_correction;
};

struct GpuDataSolverData
{
    float * Ro0, * Ro1; //cro//   //   //   //
    float * P0, * P1;   //c p//   //   //   //
    float * Vx1;
    float * Sc0, * Sc1; //   //   //   //csc//solve
    float * Cs0, * Cs1; //   //   //   //ccs//solve
    float * R10, * R11; //   //   // ro//cr1//solve
    float * G10, * G11; //   //   // fl//cg1//solve
    float * B10, * B11; //   //   //   //cb1//solve
    float * R20, * R21; //   //   //   //cr2//solve
    float * G20, * G21; //   //   //   //cg2//solve
    float * B20, * B21; //   //   //   //cb2//solve
    float * R30, * R31; //   //rro//   //cr3//solve
    float * G30, * G31; //   //r p//   //cg3//solve
    float * B30, * B31; //   //rvx//   //cb3//solve
    float dt; // шаг по времени
    int width;
    bool transponate;
};

struct GpuGravityData
{
    float * Ro0;
    float * Vx0; //cvx//   //   //   //
    float * Vy0; //cvy//   //   //   //
    float * bRo;
    float * bVx; //cvx//   //   //   //
    float * bVy; //cvy//   //   //   //
    float dt; // шаг по времени
    int width;
    int stringShift;
    int bodyCount;
};

struct GpuBodyGravityData
{
    float * bAx;
    float * bAy;
    float dt;
    int fromString;
    int toString;
    int bodyCount;
};

struct GpuFinalizeData
{
    float * Ro0, * Ro1; //cro//   //   //   //
    float * P0;   //c p//   //   //   //
    float * Vx0; //cvx//   //   //   //
    float * Vy0; //cvy//   //   //   //
    float * Vs0, *Vs1; //la2//   //   //cvs//solve
    float * Sc1; //   //   //   //csc//solve
    float * Cs1; //   //   //   //ccs//solve
    int width;
    bool transponate;
    float dt;
};

struct GpuRenderData
{
    float * Ro0; //cro//   //   //   //
    float * P0;   //c p//   //   //   //
    float * Vx0; //cvx//   //   //   //
    float * Vy0; //cvy//   //   //   //
    float * Cs0; //   //   //   //ccs//solve
    float * R10; //   //   // ro//cr1//solve
    float * G10; //   //   // fl//cg1//solve
    float * B10; //   //   //   //cb1//solve
    float * R11; //   //   // ro//cr1//solve
    float * G11; //   //   // fl//cg1//solve
    float * B11; //   //   //   //cb1//solve
    float * R20; //   //   //   //cr2//solve
    float * G20; //   //   //   //cg2//solve
    float * B20; //   //   //   //cb2//solve
    float * R30; //   //rro//   //cr3//solve
    float * G30; //   //r p//   //cg3//solve
    float * B30; //   //rvx//   //cb3//solve
    int width;
};

struct GpuCopyStringData
{
    float * data0;
    float * data1;
    float * data2;
    float * data3;
    float * data4;
    float * data5;
    float * data6;
    float * data7;
    float * data8;
    float * data9;
    float * data10;
    float * data11;
    float * data12;
    float * data13;
    float * data14;
    float * data15;
    int dstString;
    int srcString;
    int count;
};

struct GpuDtData
{
    float * data;
    int width;
};

struct GpuGravityBody
{
    float x;
    float y;
    float mass;
    float radius;
};

struct GpuData
{//                       correct  rie  solve    visc      dcr      dslv
    float * Ro0, * Ro1; // ro,cro//,   //nro, ro//   ,   //   ,   //,
    float * P0, * P1;   //  p,c p//,   // np, fl//   ,   // vp,   //,
    float * Vx0, * Vx1; // vx,cvx//,   //nvx,   //   ,   //vvx,   //,
    float * Vy0, * Vy1; // vy,cvy//,   //nvy,   //   ,   //vvy,   //,
    float * Vs0, * Vs1; // vs,cvs//,   //nvs,   //   ,   //   ,   //,
    float * Sc0, * Sc1; // sc,rsc//,   //   ,   //   ,   //   ,csc//solve,
    float * Cs0, * Cs1; // cs,La2//,   //   ,   //   ,   //   ,ccs//solve,
    float * R10, * R11; // r1,   //,   //   ,   //   ,   //   ,cr1//solve,
    float * G10, * G11; // g1,   //,   //   ,   //   ,   //   ,cg1//solve,
    float * B10, * B11; // b1,   //,   //   ,   //   ,   //   ,cb1//solve,
    float * R20, * R21; // r2,   //,   //   ,   //   ,vvx//   ,cr2//solve,
    float * G20, * G21; // g2,   //,   //   ,   //   ,vvy//   ,cg2//solve,
    float * B20, * B21; // b2,   //,   //   ,   //   , vp//   ,cb2//solve,
    float * R30, * R31; // r3,   //,rro//   ,   //   ,   //   ,cr3//solve,
    float * G30, * G31; // g3,   //,r p//   ,   //   ,   //   ,cg3//solve,
    float * B30, * B31; // b3,   //,rvx//   ,   //   ,   //   ,cb3//solve,
    float * bRo;
    float * bVx;
    float * bVy;
    float * bAx;
    float * bAy;
    GpuGravityBody * gravity;

    GpuGasCorrectData getGpuGasCorrectData(bool transponate, int width, float dt);
    GpuGasRiemanData getGpuGasRiemanData(bool transponate, int width);
    GpuGasSolverData getGpuGasSolverData(bool transponate, int width, float dt, float maxVisc);
    GpuGasViscData getGpuGasViscData(bool transponate, int width, float dt);
    GpuDataCorrectData getGpuDataCorrectData(bool transponate, int width, float sc_param_correction);
    GpuDataSolverData getGpuDataSolverData(bool transponate, int width, float dt);
    GpuGravityData getGpuGravityData(int stringShift, int bodyCount, int width, float dt);
    GpuBodyGravityData getGpuBodyGravityData(int fromString, int toString, int bodyCount, float dt);
    GpuFinalizeData getGpuFinalizeData(bool transponate, int width, float dt);
    GpuRenderData getGpuRenderData(bool transponate, int width);
    GpuCopyStringData getGpuCopyStringDataBase(int dstString, int srcString);
    GpuCopyStringData getGpuCopyStringDataCorrectGas(int dstString, int srcString);
    GpuCopyStringData getGpuCopyStringDataSolvedGas(int dstString, int srcString);
    GpuCopyStringData getGpuCopyStringDataCorrectData(int dstString, int srcString);
    GpuDtData getGpuDtData(int width);
};

#endif // GPUDATA_H
