#include "outdata.h"
#include "filemanager.h"
#include "EasyBMP/EasyBMP.h"
#include "log.h"

#include <stdlib.h>

#define min(a,b) ((a)<(b)?(a):(b))
#define max(a,b) ((a)>(b)?(a):(b))

#define limit(x,minVal,maxVal) (max(minVal,min(maxVal,x)))

bool operator<(Colour l, Colour r)
{
    return (l.R()*256*256+l.G()*256+l.B())<(r.R()*256*256+r.G()*256+r.B());
}

OutData::OutData()
{
}

bool OutData::saveDescription(std::string path, std::string bmpPath)
{
    if(_pixels.size()<1 || _pixels[0].size()<1)
    {
        logger(std::string("wrong pixels size in OutData::saveDescription"));
        return false;
    }
    std::set<Colour> colors;
    for(int i=0;i<_pixels.size();++i)
    {
        for(int j=0;j<_pixels[0].size();++j)
        {
            if(i>0&&j>0&&i<_pixels.size()-1&&j<_pixels[0].size()-1&&
                    !(_pixels[i][j]==_pixels[i-1][j])&&
                    !(_pixels[i][j]==_pixels[i+1][j])&&
                    !(_pixels[i][j]==_pixels[i][j-1])&&
                    !(_pixels[i][j]==_pixels[i][j+1]))
                continue;
            colors.insert(_pixels[i][j]);
        }
    }
    Gas gas;
    std::vector<std::string> out;
    out.push_back(std::string("pixels=")+bmpPath);
    out.push_back(std::string("texture1=")+bmpPath);
    out.push_back(std::string("texture2=")+bmpPath);
    out.push_back(std::string("texture3=")+bmpPath);
    out.push_back(std::string(""));
    out.push_back(std::string(Gas::T_RO)+";"+
                  std::string(Gas::T_PR)+";"+
                  std::string(Gas::T_VX)+";"+
                  std::string(Gas::T_VY)+";"+
                  std::string(Gas::T_VS)+";"+
                  std::string(Gas::T_CS)+";"+
                  std::string(Gas::T_SC)+";"+
                  std::string(Gas::T_FULL_TIME_LEFT)+";"+
                  std::string(Gas::T_MARK_TIME_LEFT));
    out.push_back(std::string(""));
    out.push_back(std::string("//body=")+
                  std::string(CpuGravityBody::BODY_ID)+","+
                  std::string(CpuGravityBody::BODY_X)+","+
                  std::string(CpuGravityBody::BODY_Y)+","+
                  std::string(CpuGravityBody::BODY_VX)+","+
                  std::string(CpuGravityBody::BODY_VY)+","+
                  std::string(CpuGravityBody::BODY_MASS)+","+
                  std::string(CpuGravityBody::BODY_RADIUS)+","+
                  std::string(CpuGravityBody::BODY_LIFEPATH)+","+
                  std::string(CpuGravityBody::BODY_LIFETIME)+","+
                  std::string(CpuGravityBody::BODY_BODY_V)+","+
                  std::string(CpuGravityBody::BODY_GAS_V)+","+
                  std::string(CpuGravityBody::BODY_GAS_M));
    out.push_back(std::string(""));
    for(std::set<Colour>::iterator it = colors.begin(); it!=colors.end();++it)
    {
        Colour color=*it;
        out.push_back(std::string("["+intToString(color.R()) + "," +intToString(color.G()) + ","+intToString(color.B()) + "]={}{}"));
    }

    FileManager file;
    return file.write(path,out);
}

bool readBmp(std::vector<std::vector<Colour> > * target, std::string path)
{
    BMP bmp;
    if(!bmp.ReadFromFile(path.c_str()))
    {
        logger(std::string("can't read bmp file int readBmp"));
        return false;
    }
    target->resize(bmp.TellWidth());
    for(int i=0;i<bmp.TellWidth();++i)
    {
        (*target)[i].resize(bmp.TellHeight());
        for(int j=0;j<bmp.TellHeight();++j)
        {
            RGBApixel pix=bmp.GetPixel(i,j);
            (*target)[i][j]=Colour(pix.Red,pix.Green,pix.Blue);
        }
    }
    return true;
}

SurfaceDescription makeSurfaceDescrption(std::string str, std::map<std::string,Description> & gases)
{
    SurfaceDescription out;
    std::vector<std::string> fdescription= split_string(str,'{');

    int maxLen=0;

    if(!fdescription.size())
    {
        Description description;
        out.state.resize(1);
        out.state[0].push_back(description);
    }
    else
    {
        for(std::vector<std::string>::iterator it=fdescription.begin();it!=fdescription.end();++it)
        {
            std::string xval=*it;
            xval = symbol_eraser(xval,'}');
            std::vector<std::string> xvals=split_string(xval,',');
            out.state.push_back(std::vector<Description>());
            if(!xvals.size())
            {
                Description description;
                out.state[out.state.size()-1].push_back(description);
            }
            else
            {
                for(std::vector<std::string>::iterator jt=xvals.begin();jt!=xvals.end();++jt)
                {
                    std::string fffgas = *jt;
                    Description description;
                    if(gases.count(fffgas))
                        description.addParams(gases.at(fffgas).params);
                    out.state[out.state.size()-1].push_back(description);
                }
                if(maxLen<out.state[out.state.size()-1].size())
                    maxLen=out.state[out.state.size()-1].size();
            }
        }
    }

    if(maxLen<=1)
        return out;

    for(std::vector<std::vector<Description> >::iterator it = out.state.begin(); it!=out.state.end(); ++it)
    {
        std::vector<Description> stDesc = *it;
        if(stDesc.size()<maxLen)
        {
            std::vector<Description> nDesc;
            float step=1.0/(maxLen-1);

            for(int i=0;i<maxLen;++i)
            {
                float pos=i*step;
                float oldPos=pos*(stDesc.size()-1);
                int pre=oldPos;
                int ne=oldPos+1;
                float lcoe=oldPos-pre;
                float rcoe=1.0-lcoe;
                Description l=stDesc[pre];
                Description r=stDesc[ne];
                Description m;
                for(std::map<std::string,float>::iterator jt = l.params.begin(); jt!=l.params.end();++jt)
                {
                    float val=lcoe*(l.params.count(jt->first)?l.params[jt->first]:0.0)+rcoe*(r.params.count(jt->first)?r.params[jt->first]:0.0);
                    m.params[jt->first]=val;
                }
                for(std::map<std::string,float>::iterator jt = r.params.begin(); jt!=r.params.end();++jt)
                {
                    if(m.params.count(jt->first))
                       continue;
                    float val=lcoe*(l.params.count(jt->first)?l.params[jt->first]:0.0)+rcoe*(r.params.count(jt->first)?r.params[jt->first]:0.0);
                    m.params[jt->first]=val;
                }
                nDesc.push_back(m);
            }
            *it = nDesc;
        }
    }



    return out;
}

Description makeDescription(std::string str,  std::map<std::string,float> & defaultGas, std::set<std::string> & gasParams)
{
    Description out;
    str=symbol_eraser(str,'}');
    str=symbol_eraser(str,'{');
    std::vector<std::string> params = split_string(str,',');
    for(std::map<std::string,float>::iterator it = defaultGas.begin();it!=defaultGas.end();++it)
    {
        out.set(it->first,it->second);
    }
    for(std::vector<std::string>::iterator it = params.begin();it!=params.end();++it)
    {
        std::string param = *it;
        std::vector<std::string> fparams=split_string(param,'=');
        if(fparams.size()==2 && gasParams.count(fparams[0]))
        {
            out.set(fparams[0],stringToFloat(fparams[1]));
        }
    }
    return out;
}

bool OutData::loadDescription(std::string path)
{
    FileManager file;
    std::vector<std::string> out = file.read(path);
    if(out.size()<1)
    {
        logger(std::string("empty description in OutData::loadDescription"));
        return false;
    }
    std::set<std::string> gasParams;
    gasParams.insert(std::string(Gas::T_RO));
    gasParams.insert(std::string(Gas::T_PR));
    gasParams.insert(std::string(Gas::T_VX));
    gasParams.insert(std::string(Gas::T_VY));
    gasParams.insert(std::string(Gas::T_VS));
    gasParams.insert(std::string(Gas::T_CS));
    gasParams.insert(std::string(Gas::T_SC));
    gasParams.insert(std::string(Gas::T_FULL_TIME_LEFT));
    gasParams.insert(std::string(Gas::T_MARK_TIME_LEFT));
    std::map<std::string,float> defaultGas;
    std::map<std::string,Description> gases;
    for(std::vector<std::string>::iterator it = out.begin();it!=out.end();++it)
    {
        std::string str=*it;
        std::vector<std::string> fsplit = split_string(str,'=',true);
        if(fsplit.size()==2)
        {
            std::string fpar=fsplit[0];
            std::string fval=fsplit[1];
            if(fpar=="pixels")
            {
                if(!readBmp(&_pixels,fval))
                    return false;
            }
            else if(fpar=="texture1")
            {
                if(!readBmp(&_texture1,fval))
                    return false;
            }
            else if(fpar=="texture2")
            {
                if(!readBmp(&_texture2,fval))
                    return false;
            }
            else if(fpar=="texture3")
            {
                if(!readBmp(&_texture3,fval))
                    return false;
            }
            else if(fpar=="body")
            {
                std::vector<std::string> fparams=split_string(fval,',');
                std::map<std::string,std::string> params;
                for(int i=0;i<fparams.size();++i)
                {
                    std::vector<std::string> ffp=split_string(fparams[i],'=');
                    if(ffp.size()==2)
                    {
                        params[ffp[0]]=ffp[1];
                    }
                }
                CpuGravityBody body("");
                for(std::map<std::string,std::string>::iterator it=params.begin();it!=params.end();++it)
                {
                    if(it->first==CpuGravityBody::BODY_ID)
                    {
                        body.id=it->second;
                    }
                    else if(it->first==CpuGravityBody::BODY_X)
                    {
                        body.x=stringToFloat(it->second);
                    }
                    else if(it->first==CpuGravityBody::BODY_Y)
                    {
                        body.y=stringToFloat(it->second);
                    }
                    else if(it->first==CpuGravityBody::BODY_VX)
                    {
                        body.vx=stringToFloat(it->second);
                    }
                    else if(it->first==CpuGravityBody::BODY_VY)
                    {
                        body.vx=stringToFloat(it->second);
                    }
                    else if(it->first==CpuGravityBody::BODY_MASS)
                    {
                        body.mass=stringToFloat(it->second);
                    }
                    else if(it->first==CpuGravityBody::BODY_RADIUS)
                    {
                        body.radius=stringToFloat(it->second);
                    }
                    else if(it->first==CpuGravityBody::BODY_LIFEPATH)
                    {
                        body.lifePath=stringToFloat(it->second);
                    }
                    else if(it->first==CpuGravityBody::BODY_LIFETIME)
                    {
                        body.lifeTime=stringToFloat(it->second);
                    }
                    else if(it->first==CpuGravityBody::BODY_BODY_V)
                    {
                        body.bodyV=stringToFloat(it->second);
                    }
                    else if(it->first==CpuGravityBody::BODY_GAS_V)
                    {
                        body.gasV=stringToFloat(it->second);
                    }
                    else if(it->first==CpuGravityBody::BODY_GAS_M)
                    {
                        body.gasM=stringToFloat(it->second);
                    }
                }

                if(body.id!="")
                    body.id=intToString(rand());

                _body.push_back(body);
            }
            else if(fpar.find_first_of('[')!=std::string::npos)
            {
                fpar=symbol_eraser(fpar,'[');
                fpar=symbol_eraser(fpar,']');
                std::vector<std::string> ffcolors=split_string(fpar,',');
                if(ffcolors.size()!=3)
                    return false;
                Colour color(stringToInt(ffcolors[0]),stringToInt(ffcolors[1]),stringToInt(ffcolors[2]));
                if(color.R()+color.G()+color.B()==0)
                    continue;
                SurfaceDescription description=makeSurfaceDescrption(fval,gases);
                _descriptions[color]=description;
            }
            else if(gasParams.count(fpar))
            {
                defaultGas[fpar]=stringToFloat(fval);
            }
            else if(fpar.size()>0)
            {
                gases[fpar]=makeDescription(fval,defaultGas,gasParams);
            }
        }
        else
        {
            if(str.find_first_of('!')!=std::string::npos)
            {
                str=symbol_eraser(str,'!');
                defaultGas.erase(str);
            }
            else if(str.size()>0)
            {
                gases[symbol_eraser(str,'=')]=makeDescription("",defaultGas,gasParams);
            }
        }
    }
    return true;
}

bool OutData::loadPicture(std::string path, int texNum)
{
    BMP picture;
    std::vector<std::vector<Colour> > * pixels = &_pixels;

    if(texNum==1)
        pixels=&_texture1;
    if(texNum==2)
        pixels=&_texture2;
    if(texNum==3)
        pixels=&_texture3;

    if(!picture.ReadFromFile(path.c_str()))
    {
        logger(std::string("can't read in OutData::loadPicture from file : ")+path);
        return false;
    }

    pixels->resize(picture.TellWidth());

    for(int i = 0; i < picture.TellWidth(); ++i)
    {
        for(int j=0; j < picture.TellHeight(); ++j)
            (*pixels)[i].push_back(Colour(picture.GetPixel(i,j).Red,picture.GetPixel(i,j).Green,picture.GetPixel(i,j).Blue));
    }

    return true;
}

bool OutData::open(std::string path, Position shift)
{
    std::string picture;
    std::string description;
    _shift=shift;
    if(path.find(".bmp",0,4)!=std::string::npos)
    {
        picture = path;
        description=path;
        description = description.replace(path.find(".bmp",0,4), std::string::npos,".xml");
        if(!loadPicture(picture))
        {
            logger(std::string("can't load picture int OutData::open from file : ")+path);
            return false;
        }
        saveDescription(description,picture);
    }
    else if(path.find(".xml",0,4)!=std::string::npos||path.find(".txt",0,4)!=std::string::npos)
    {
        description = path;
        picture = path;
        if(path.find(".xml",0,4)!=std::string::npos)
            picture = picture.replace(path.find(".xml",0,4), std::string::npos,".bmp");
        if(path.find(".txt",0,4)!=std::string::npos)
            picture = picture.replace(path.find(".txt",0,4), std::string::npos,".bmp");
        if(!loadDescription(description))
        {
            logger(std::string("can't load description in OutData::open from file : ")+path);
            return false;
        }
    }
    else
    {
        logger(std::string("can't load unknown in OutData::open from file : ")+path);
        return false;
    }
    return true;
}

Gas OutData::update(Gas gas,Position position)
{   
    if(_pixels.size()<1||_pixels[0].size()<1)
    {
        logger(std::string("wrong pixels size in OutData::update"));
        return gas;
    }
    position.x+=_pixels.size()/2-_shift.x;
    position.y+=_pixels[0].size()/2-_shift.y;
    if(position.x<0 || position.y<0 || position.x>=_pixels.size() || position.y>=_pixels[0].size())
        return gas;
    Colour color = _pixels[position.x][position.y];
    if(color.R()+color.G()+color.B()==0)
        return gas;


    if(_texture1.size()>0 && _texture1[0].size()>0)
    {
        Position textPos(_texture1.size()-_pixels.size(),_texture1[0].size()-_pixels[0].size());
        textPos=position+textPos;
        if(textPos.x>=0&&textPos.y>=0&&textPos.x<_texture1.size()&&textPos.y<_texture1[0].size())
        {
            Colour texColor = _texture1[textPos.x][textPos.y];
            gas.R1=texColor.R();
            gas.G1=texColor.G();
            gas.B1=texColor.B();
        }
    }

    if(_texture2.size()>0 && _texture2[0].size()>0)
    {
        Position textPos(_texture2.size()-_pixels.size(),_texture2[0].size()-_pixels[0].size());
        textPos=position+textPos;
        if(textPos.x>=0&&textPos.y>=0&&textPos.x<_texture2.size()&&textPos.y<_texture2[0].size())
        {
            Colour texColor = _texture2[textPos.x][textPos.y];
            gas.R2=texColor.R();
            gas.G2=texColor.G();
            gas.B2=texColor.B();
        }
    }

    if(_texture3.size()>0 && _texture3[0].size()>0)
    {
        Position textPos(_texture3.size()-_pixels.size(),_texture3[0].size()-_pixels[0].size());
        textPos=position+textPos;
        if(textPos.x>=0&&textPos.y>=0&&textPos.x<_texture3.size()&&textPos.y<_texture3[0].size())
        {
            Colour texColor = _texture3[textPos.x][textPos.y];
            gas.R3=texColor.R();
            gas.G3=texColor.G();
            gas.B3=texColor.B();
        }
    }

    if(_descriptions.count(color)==0)
        return gas;
    SurfaceDescription surface=_descriptions.at(color);
    if(surface.size.x+surface.size.y<0)
    {
        surface.position.x=_pixels.size()+1;
        surface.position.y=_pixels[0].size()+1;
        for(int i=0;i<_pixels.size();i++)
        {
            for(int j=0;j<_pixels[0].size();j++)
            {
                if(_pixels[i][j]==color)
                {
                    if(surface.position.x>i)
                       surface.position.x=i;
                    if(surface.position.y>j)
                       surface.position.y=j;
                    if(surface.size.x<i)
                        surface.size.x=i;
                    if(surface.size.y<j)
                        surface.size.y=j;
                }
            }
        }
        surface.size.x = surface.size.x-surface.position.x+1;
        surface.size.y = surface.size.y-surface.position.y+1;
        _descriptions[color]=surface;
    }

    Description r;

    if(surface.state.size()>1)
    {
        float posx=(float(position.x-surface.position.x))/surface.size.x;
        float posy=(float(position.y-surface.position.y))/surface.size.y;
        float fx=posx*(surface.state[0].size()-1);
        float fy=posy*(surface.state.size()-1);
        int x = fx;
        int y = fy;

        if(x<0)
            x=0;
        if(y<0)
            y=0;
        if(x>surface.state[0].size()-1)
            x=surface.state[0].size()-1;
        if(y>surface.state.size()-1)
            y=surface.state.size()-1;
        int nx=min(x+1,surface.state[x].size()-1);
        int ny=min(y+1,surface.state.size()-1);

        Description c[4];
        c[0]=surface.state[y][x];
        c[1]=surface.state[y][nx];
        c[2]=surface.state[ny][x];
        c[3]=surface.state[ny][nx];

        for(int n=0;n<4;n++)
        {
            Description cur=c[n];
            for(std::map<std::string,float>::iterator jt = cur.params.begin(); jt!=cur.params.end();++jt)
            {
                if(r.params.count(jt->first))
                   continue;

                float cx=fx-x;
                float cy=fy-y;

                float val=(1.0-cx)*(1.0-cy)*(c[0].params.count(jt->first)?c[0].params[jt->first]:0.0)+
                        cx*(1.0-cy)*(c[1].params.count(jt->first)?c[1].params[jt->first]:0.0)+
                        (1.0-cx)*cy*(c[2].params.count(jt->first)?c[2].params[jt->first]:0.0)+
                        cx*cy*(c[3].params.count(jt->first)?c[3].params[jt->first]:0.0);
                r.params[jt->first]=val;
            }
        }
    }
    else
    {

        float dist=sqrt((surface.position.x+surface.size.x/2-position.x)*
                         (surface.position.x+surface.size.x/2-position.x)+
                         (surface.position.y+surface.size.y/2-position.y)*
                         (surface.position.y+surface.size.y/2-position.y));
        float dx=-(surface.position.x+surface.size.x/2-position.x)/max(dist,1e-6);
        float dy=-(surface.position.y+surface.size.y/2-position.y)/max(dist,1e-6);


        float f=min(1.0,dist*2/max(1e-6,min(surface.size.x,surface.size.y)))*(surface.state[0].size()-1);
        int p = f;
        if(p<0)
            p=0;
        if(p>surface.state[0].size()-1)
            p=surface.state[0].size()-1;
        int np=min(p+1,surface.state[0].size()-1);

        Description c[2];
        c[0]=surface.state[0][p];
        c[1]=surface.state[0][np];

        for(int n=0;n<2;n++)
        {
            Description cur=c[n];
            for(std::map<std::string,float>::iterator jt = cur.params.begin(); jt!=cur.params.end();++jt)
            {
                if(r.params.count(jt->first))
                   continue;

                float cp=f-p;

                if(jt->first==Gas::T_VX||jt->first==Gas::T_VY)
                {
                    float val=(1-cp)*(c[0].params.count(Gas::T_VX)?c[0].params[Gas::T_VX]:0.0)+
                                            cp*(c[1].params.count(Gas::T_VX)?c[1].params[Gas::T_VX]:0.0);

                    float nval=(1-cp)*(c[0].params.count(Gas::T_VY)?c[0].params[Gas::T_VY]:0.0)+
                            cp*(c[1].params.count(Gas::T_VY)?c[1].params[Gas::T_VY]:0.0);

                    float vx=(dx*val - dy*nval)*dist*2/(surface.size.x+surface.size.y);
                    float vy=(dy*val + dx*nval)*dist*2/(surface.size.x+surface.size.y);

                    r.params[Gas::T_VX]=vx;
                    r.params[Gas::T_VY]=vy;

                    continue;
                }

                float val=(1-cp)*(c[0].params.count(jt->first)?c[0].params[jt->first]:0.0)+
                        cp*(c[1].params.count(jt->first)?c[1].params[jt->first]:0.0);

                r.params[jt->first]=val;
            }
        }
    }

    for(std::map<std::string,float>::iterator it=r.params.begin();it!=r.params.end();++it)
    {
        if(it->first==Gas::T_RO)
        {
            gas.Ro=limit(it->second,-1024*1024,1024*1024);
        }
        else if(it->first==Gas::T_PR)
        {
            gas.P=it->second;
        }
        else if(it->first==Gas::T_VX)
        {
            gas.Vx=limit(it->second,-1024.0,1024.0);
        }
        else if(it->first==Gas::T_VY)
        {
            gas.Vy=limit(it->second,-1024.0,1024.0);
        }
        else if(it->first==Gas::T_VS)
        {
            gas.Vs=it->second;
        }
        else if(it->first==Gas::T_CS)
        {
            gas.Cs=it->second;
        }
        else if(it->first==Gas::T_SC)
        {
            gas.Sc=it->second;
        }
        else if(it->first==Gas::T_FULL_TIME_LEFT)
        {
            gas.FullTimeLeft=it->second;
        }
        else if(it->first==Gas::T_MARK_TIME_LEFT)
        {
            gas.MarkTimeLeft=it->second;
        }
        else
        {
            logger(std::string("unknown variable in OutData::update : "+it->first));
        }
    }
    gas.P=limit(gas.P,-fabs(gas.Ro)*1024*1024,fabs(gas.Ro)*1024*1024);
    gas.Vs=limit(gas.Vs,0,fabs(gas.Ro)*1024.0);
    gas.check();

//    if(color==Colour(0,0,128))
//    {
//        int aaa=0;
//        ++aaa;
//    }

    return gas;
}

std::vector<CpuGravityBody> OutData::getBodys(Position zeroPosition)
{
    std::vector<CpuGravityBody> bodys;

    for(int i=0;i<_body.size();++i)
    {
        CpuGravityBody body=_body[i];
        body.x=body.x+0.5f-zeroPosition.x+_shift.x;
        body.y=body.y+0.5f-zeroPosition.y+_shift.y;
        bodys.push_back(body);
    }

    return bodys;
}
