#ifndef OUTDATA_H
#define OUTDATA_H

#include <string>
#include <vector>
#include <map>
#include "cpudata.h"

struct Colour
{
    unsigned char r;
    unsigned char g;
    unsigned char b;
    Colour():r(0),g(0),b(0){}
    Colour(int R,int G, int B)
    {
        r=static_cast<unsigned char>(R);
        g=static_cast<unsigned char>(G);
        b=static_cast<unsigned char>(B);
    }
    inline int R(){return static_cast<int>(r);}
    inline int G(){return static_cast<int>(g);}
    inline int B(){return static_cast<int>(b);}
    bool operator==(Colour out){return (r==out.r)&&(g==out.g)&&(b==out.b);}
    bool operator<(Colour out){return (R()*256*256+G()*256+B())<(out.R()*256*256+out.G()*256+out.B());}
};

bool operator<(Colour l, Colour r);


struct Description
{
    std::map<std::string,float> params;
    void set(std::string key, float value){params[key]=value;}
    bool check(std::string key){return params.count(key)>0;}
    float get(std::string key){return check(key)?params.at(key):0;}
    void addParams(std::map<std::string,float> input)
    {
        for(std::map<std::string,float>::iterator it= input.begin();it!=input.end();++it)
        {
            if(params[it->first]==100)
            {
                int aaa=0;
                aaa++;
            }
            set(it->first,it->second);
        }
    }
};

struct SurfaceDescription
{
    Position position;//left up angle
    Position size;//
    SurfaceDescription():position(-1,-1),size(-1,-1){}
    std::vector<std::vector<Description> > state;
    Gas update(Gas gas,Position position, Position surface,Position surfaceSize);
};

class OutData
{
    std::vector<std::vector<Colour> > _pixels;
    std::vector<std::vector<Colour> > _texture1;
    std::vector<std::vector<Colour> > _texture2;
    std::vector<std::vector<Colour> > _texture3;
    std::map<Colour,SurfaceDescription> _descriptions;
    Position _shift;

    std::vector<CpuGravityBody> _body;

    bool saveDescription(std::string path, std::string bmpPath);
    bool loadDescription(std::string path);
    bool loadPicture(std::string path, int texNum=0);

public:
    OutData();

    bool open(std::string path,Position shift);
    Gas update(Gas gas,Position position);
    std::vector<CpuGravityBody> getBodys(Position zeroPosition);
};

#endif // OUTDATA_H
