#include "output.h"
#include "log.h"
#include "EasyBMP/EasyBMP.h"
#include <vector>
#include <string>
#include <sstream>

#define min(a,b) ((a)<(b)?(a):(b))
#define max(a,b) ((a)>(b)?(a):(b))

#define SAFE_CALL(function,function_description,description)\
error = function;\
if(error)\
{\
    std::cout<<error<<" error in "<<function_description<<" in "<<description<<std::endl;\
}

std::string buildFilePath(std::string path, int number)
{
    std::stringstream ss;
    ss << number;
    std::string str = ss.str();
    for(int i=str.size();i<8;++i)
        path += '0';
    path += str + std::string(".bmp");
    return path;
}

void * OutPut::thread_save(void * arg)
{
    OutPut * _this = (OutPut *)arg;
    int error;
    SAFE_CALL(pthread_mutex_lock(&_this->_mutex),"pthread_mutex_lock","Device::thread_save")
    _this->_whait_for_mutex=false;

    Camera camera = _this->_scenario->getCamera();
    std::vector<std::vector<float> > rgb = camera.getFrame(_this->_rgb,_this->_cpuData->getSize().x,_this->_cpuData->getSize().y);

    std::string path = buildFilePath(_this->_scenario->getOutputPath(),_this->_frame++);

    SAFE_CALL(pthread_mutex_unlock(&_this->_mutex),"pthread_mutex_UNlock","Device::thread_stepTo")

    BMP bmp;

    bmp.SetSize(camera.size.x,camera.size.y);

    for(int i=0;i<camera.size.x;++i)
    {
        for(int j=0;j<camera.size.y;++j)
        {
            RGBApixel color;
            color.Alpha=255;
            int pos=j*camera.size.x+i;
            color.Red=static_cast<ebmpBYTE>(min(255.1,max(0.1,rgb[0][pos])));
            color.Green=static_cast<ebmpBYTE>(min(255.1,max(0.1,rgb[1][pos])));
            color.Blue=static_cast<ebmpBYTE>(min(255.1,max(0.1,rgb[2][pos])));
            bmp.SetPixel(i,j,color);
        }
    }

    if(!bmp.WriteToFile(path.c_str()))
    {
        logger(std::string("can't write bmp file in : OutPut::save"+path));
        pthread_exit(NULL);
    }

    pthread_exit(NULL);
}

void OutPut::save()
{
    int error;
    _whait_for_mutex=true;
    SAFE_CALL(pthread_create(&_thread, &_attr, thread_save, (void *)this),"pthread_create","Device::stepTo")
    while(_whait_for_mutex)
    {

    }
}

bool OutPut::init(CpuData * cpuData, Scenario * scenario)
{
    _cpuData=cpuData;
    if(!_cpuData)
    {
        logger("no cpudata in OutPut::init");
        return false;
    }
    if(!scenario)
    {
        logger("no scenario in OutPut::init");
        return false;
    }
    _frame=0;
    std::string path = buildFilePath(scenario->getOutputPath(),_frame);
    FILE * file = fopen(path.c_str(),"rb");
    while(file)
    {
        fclose(file);
        path = buildFilePath(scenario->getOutputPath(),++_frame);
        file = fopen(path.c_str(),"rb");
    }
    _scenario= scenario;
    _rgb.resize(3);
    _rgb[0].resize(_cpuData->getSize().x*_cpuData->getSize().y);
    _rgb[1].resize(_cpuData->getSize().x*_cpuData->getSize().y);
    _rgb[2].resize(_cpuData->getSize().x*_cpuData->getSize().y);
    return true;
}


std::vector<std::vector<float> > & OutPut::getColors()
{
    return _rgb;
}

OutPut::OutPut()
{
    int error;
    SAFE_CALL(pthread_attr_init( &_attr ),"pthread_attr_init","Device::init")
    SAFE_CALL(pthread_attr_setdetachstate( &_attr, PTHREAD_CREATE_DETACHED ),"pthread_attr_setdetachstate","Device::init")
    SAFE_CALL(pthread_mutex_init(&_mutex, NULL),"pthread_mutex_init","Device::init")
}

OutPut::~OutPut()
{
    int error;
    SAFE_CALL(pthread_attr_destroy(&_attr),"pthread_attr_destroy","Device::~Device")
    SAFE_CALL(pthread_mutex_destroy(&_mutex),"pthread_mutex_destroy","Device::~Device")
}

int OutPut::getFrame()
{
    return _frame;
}


