#ifndef SCENARIO_H
#define SCENARIO_H

#include "outdata.h"
#include "cpudata.h"
#include "gpudata.h"
#include "filemanager.h"

struct Camera
{
    static const char * CAMERA_POSITION_X;
    static const char * CAMERA_POSITION_Y;
    Position position;
    static const char * CAMERA_SIZE_X;
    static const char * CAMERA_SIZE_Y;
    Position size;
    Camera():size(1920,1080){}
    std::vector<std::vector<float> > getFrame(std::vector<std::vector<float> > & rgb, int xsize, int ysize);
};

struct UniPointer
{
    float * f;
    int * i;
    std::string * s;
    UniPointer():f(0),i(0),s(0){}
    UniPointer(float * p):f(p),i(0),s(0){}
    UniPointer(int * p):f(0),i(p),s(0){}
    UniPointer(std::string * p):f(0),i(0),s(p){}
    void operator=(long v){if(i) *i=static_cast<int>(v); else if(f) *f=static_cast<float>(v);}
    void operator=(float v){if(f) *f=static_cast<float>(v); else if(i) *i=static_cast<int>(v);}
    void operator=(std::string v){if(s) *s=v;}
    bool isInt(){return i!=0;}
    bool isFloat(){return f!=0;}
    bool isString(){return s!=0;}
    float getFloat(){if(f) return *f; else return 0;}
    int getInt(){if(i) return *i; return 0;}
    std::string getString(){ if(s) return *s; else return "";}
};

class Scenario
{
    std::vector<GpuInfo> _gpuInfo;
    Camera _camera;

    static const char * SCENARIO_FULL_SIZE_X;
    static const char * SCENARIO_FULL_SIZE_Y;
    Position _fullSize;

    static const char * SCENARIO_TOTAL_TIME;
    float _totalTime;
    static const char * SCENARIO_TOTAL_STEP;
    int _totalStep;
    static const char * SCENARIO_ENTERFRAME;
    int _enterFrame;
    static const char * SCENARIO_TIMEFRAME;
    float _timeFrame;
    static const char * SCENARIO_ENTERSAVE;
    int _enterSave;

    static const char * SCENARIO_SCHEME_CORREECTION;
    float _schemeCorection;
    static const char * SCENARIO_MAX_VISC;
    float _maxVisc;


    static const char * SCENARIO_OUTPUT_PATH;
    std::string _outputPath;

    bool _loadDefault(std::string pathToApplication);
    bool _saveDefault(std::string pathToApplication);

    std::string _path;
    std::vector<std::string> _commands;

    float _sleepTime;

    std::map<std::string,UniPointer> _getParams();
    bool _setParams(std::vector<std::string> input);


    float _lastUpdateTime;

public:
    Scenario(std::string pathToApplication);
    bool init(std::string path);
    bool init(CpuData & cpuData);
    bool update(CpuData & cpuData, std::vector<Device> * gpuData=0);
    void refresh(bool getParams=true);
    bool inProgress(CpuData & cpuData);
    bool makeVoid(std::string path, std::string firstElement);

    float getTotalTime();
    int getTotalStep();
    int getEnterFrame();
    float getTimeFrame();
    int getEnterSave();
    std::vector<GpuInfo> getGpuInfo();
    Camera getCamera();
    std::string getOutputPath();
    std::string getPath();
    Point getGravity();
    float getParamsSchemeCorrection();
    float getMaxVisc();
    bool save(CpuData & cpuData, std::string path);
    std::map<int,int> buildGpuStrings(Position size);
};

#endif // SCENARIO_H
