#include <iostream>

#include "simulation.h"
#include "output.h"
#include <math.h>
#include <pthread.h>
#include "log.h"

Simulation::Simulation(std::string pathToApplication):_scenario(pathToApplication)
{

}

bool Simulation::command(std::string command)
{
    if(command.find("bmp",0,3)!=std::string::npos)
    {
        OutData outdata;
        if(!outdata.open(command,Position(0,0)))
        {
            std::cout<<"can't open "<< command <<std::endl;
        }
        std::cout<<"make new description file for "<< command << std::endl;

    }
    else if(command.find("xml",0,3)!=std::string::npos)
    {
        std::string description = command;
        description.replace(command.find("xml",0,3), std::string::npos,"txt");
        _scenario.makeVoid(description,command);
        std::cout<<"make new scenario file "<< description << std::endl;
    }
    else if(command=="gpu"||command=="info"||command=="i")
    {
        std::vector<std::string> data = Device::getGpusInfo();
        for(std::vector<std::string>::iterator it=data.begin();it!=data.end();++it)
        {
            std::cout<<*it<<std::endl;
        }
    }
    else if(command.find("txt",0,3)!=std::string::npos)
    {
        return _run(command);
    }
    else if(command.find_first_of('.')==std::string::npos)
    {
        std::vector<std::string> fc=split_string(command,',');
        std::string path;
        int scale=1;
        int framerate=25;
        bool deleteBmp=false;
        for(int i=0;i<fc.size();++i)
        {
            std::string it=fc[i];
            if(it.find_first_of('/')!=std::string::npos)
            {
                if(it.find_last_of('/')!=it.size()-1)
                    it=it+'/';
                path=it;
            }
            if(it.find_first_of('f')!=std::string::npos)
                framerate=stringToInt(split_string(it,'=')[1]);
            if(it.find_first_of('s')!=std::string::npos)
                scale=stringToInt(split_string(it,'=')[1]);
            if(it=="d")
                deleteBmp=true;
        }
        return encoder(path,framerate,scale,deleteBmp);
    }
    else if(command!="")
    {
        std::cout<<"unknown command"<< command << std::endl;
    }
    return false;
}

bool Simulation::_run(std::string scenario)
{
    std::cout<<"starting scenario : "<<scenario<<std::endl;
    std::cout<<"scenario... ";
    std::cout.flush();
    if(!_scenario.init(scenario))
    {
        std::cout<<"can't init scenario"<<std::endl;
        return false;
    }
    std::cout<<" ok"<<std::endl;
    std::cout<<"data... ";
    std::cout.flush();
    if(!_scenario.init(_cpuData))
    {
        std::cout<<"can't init cpuData"<<std::endl;
        return false;
    }
    std::cout<<"ok"<<std::endl;
    if(!_output.init(&_cpuData,&_scenario))
    {
        std::cout<<"can't init output"<<std::endl;
        return false;
    }

    _devices.resize(_scenario.getGpuInfo().size());

    Device::setDeviceCount(_devices.size());

    std::map<int,int> deviceStrings=_scenario.buildGpuStrings(_cpuData.getSize());

    int firstLine=0;
    int nextLine=0;

    for(int d=0;d<_devices.size();++d)
    {
        nextLine=firstLine+deviceStrings[_scenario.getGpuInfo()[d].gpuId]*32;
        std::cout<<"gpu device "<<_scenario.getGpuInfo()[d].gpuId<<" ... ";
        std::cout.flush();
        if(!_devices[d].init(&_cpuData,&_scenario,_scenario.getGpuInfo()[d].gpuId,Position(firstLine,nextLine), d-1<0?0:&_devices[d-1], d+1>=_devices.size()?0:&_devices[d+1]))
        {
            std::cout<<"can't init gpu device "<<_scenario.getGpuInfo()[d].gpuId<<std::endl;
            return false;
        }
        firstLine=nextLine;
        std::cout<<"ok"<<std::endl;
    }

    while(_scenario.inProgress(_cpuData))
    {
        if(!_step())
            return false;
    }
    return true;
}

bool Simulation::_step()
{
    for(int d=0;d<_devices.size();d++)
    {
        _devices[d].getColors(_output.getColors());
    }

    _output.save();

    _cpuData.makeFrame(_scenario.getTimeFrame(), _scenario.getEnterFrame());

    for(int d=0;d<_devices.size();d++)
    {
        _devices[d].step();
    }
    for(int d=0;d<_devices.size();d++)
    {
        _devices[d].wait();
    }

    if((!(_output.getFrame()%_scenario.getEnterSave()))&&_output.getFrame())
    {
        for(int d=0;d<_devices.size();d++)
        {
            _cpuData=&_devices[d];
        }
        _scenario.save(_cpuData,_scenario.getPath()+"autosave");
    }

    std::cout<<"FRAME "<<_output.getFrame()<<std::endl;

    _scenario.refresh();

    if(!_scenario.update(_cpuData,&_devices))
    {
        logger("cant udate scenario in Simulation::_step");
        return false;
    }

    return true;
}
